#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" wicd - wireless connection daemon implementation.

This module implements the wicd daemon that provides network
connection management, for both wireless and wired networks. The daemon
must be run as root to control the networks, however the user interface
components should be run as a normal user.

class WicdDaemon() -- DBus interface to manage general wicd processes.
class WiredDaemon() -- DBus interface to managed the wired network.
class WirelessDaemon() -- DBus interface to managed the wireless network.

"""

#
#   Copyright (C) 2007 - 2009 Adam Blackburn
#   Copyright (C) 2007 - 2009 Dan O'Reilly
#   Copyright (C) 2007 - 2009 Byron Hillis
#   Copyright (C)        2009 Andrew Psaltis
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import shutil
import sys
import time
import getopt
import signal
import atexit
from subprocess import Popen

# DBUS
import gobject
import dbus
import dbus.service
if getattr(dbus, 'version', (0, 0, 0)) < (0, 80, 0):
    import dbus.glib
else:
    from dbus.mainloop.glib import DBusGMainLoop
    DBusGMainLoop(set_as_default=True)

# wicd specific libraries
from wicd import wpath
from wicd import networking
from wicd import misc
from wicd import wnettools
from wicd.misc import noneToBlankString, _status_dict
from wicd.logfile import ManagedStdio
from wicd.configmanager import ConfigManager

if __name__ == '__main__':
    wpath.chdir(__file__)

misc.RenameProcess("wicd")

wireless_conf = os.path.join(wpath.etc, "wireless-settings.conf")
wired_conf = os.path.join(wpath.etc, "wired-settings.conf")
dhclient_conf = os.path.join(wpath.etc, "dhclient.conf.template")

class WicdDaemon(dbus.service.Object):
    """ The main wicd daemon class.

    This class mostly contains exported DBus methods that are not
    associated directly with either wired or wireless actions.  There
    are a few exceptions to this, due to architectural limitations.

    """
    def __init__(self, bus_name, object_path="/org/wicd/daemon",
                 auto_connect=True):
        """ Initializes the daemon DBus object. """
        dbus.service.Object.__init__(self, bus_name=bus_name, 
                                     object_path=object_path)
        self.config = ConfigManager(os.path.join(wpath.etc,
                                                 "manager-settings.conf"))
        self._debug_mode = bool(self.config.get("Settings", "debug_mode"))
        self.wifi = networking.Wireless(debug=self._debug_mode)
        self.wired = networking.Wired(debug=self._debug_mode)
        self.wired_bus = WiredDaemon(bus_name, self, wired=self.wired)
        self.wireless_bus = WirelessDaemon(bus_name, self, wifi=self.wifi)
        self.forced_disconnect = False
        self.need_profile_chooser = False
        self.current_interface = None
        self.vpn_session =  None
        self.gui_open = False
        self.suspended = False
        self._debug_mode = False
        self.connection_state = misc.NOT_CONNECTED
        self.connection_info = [""]
        self.auto_connecting = False
        self.prefer_wired = False
        self.show_never_connect = True
        self.dhcp_client = 0
        self.link_detect_tool = 0
        self.flush_tool = 0
        self.sudo_app = 0

        # This will speed up the scanning process - if a client doesn't 
        # need a fresh scan, just feed them the old one.  A fresh scan
        # can be done by calling Scan(fresh=True).
        self.LastScan = []

        # Load the config file
        self.ReadConfig()

        signal.signal(signal.SIGTERM, self.DaemonClosing)
        self.DaemonStarting()

        # Scan since we just got started
        if not auto_connect:
            print "--no-autoconnect detected, not autoconnecting..."
            self.SetForcedDisconnect(True)
        self.wireless_bus.Scan()

    def get_debug_mode(self):
        return self._debug_mode
    def set_debug_mode(self, mode):
        self._debug_mode = mode
        self.config.debug = mode
    debug_mode = property(get_debug_mode, set_debug_mode)

    @dbus.service.method('org.wicd.daemon')
    def Hello(self):
        """ Returns the version number. 

        This number is major-minor-micro. Major is only incremented if minor
        reaches > 9. Minor is incremented if changes that break core stucture
        are implemented. Micro is for everything else, and micro may be
        anything >= 0. This number is effective starting wicd v1.2.0.

        """
        return wpath.version

    @dbus.service.method('org.wicd.daemon')
    def SetWiredInterface(self, interface):
        """ Sets the wired interface for the daemon to use. """
        print "setting wired interface %s" % (str(interface))
        self.wired.wired_interface = noneToBlankString(interface)
        self.config.set("Settings", "wired_interface", interface, write=True)

    @dbus.service.method('org.wicd.daemon')
    def SetWirelessInterface(self, interface):
        """ Sets the wireless interface the daemon will use. """
        print "setting wireless interface %s" % (str(interface))
        self.wifi.wireless_interface = noneToBlankString(interface)
        self.config.set("Settings", "wireless_interface", interface, write=True)

    @dbus.service.method('org.wicd.daemon')
    def SetWPADriver(self, driver):
        """ Sets the wpa driver the wpa_supplicant will use. """
        print "setting wpa driver", str(driver)
        self.wifi.wpa_driver = driver
        self.config.set("Settings", "wpa_driver", driver, write=True)

    @dbus.service.method('org.wicd.daemon')
    def SetUseGlobalDNS(self, use):
        """ Sets a boolean which determines if global DNS is enabled. """
        print 'setting use global dns to', use
        use = misc.to_bool(use)
        self.config.set("Settings", "use_global_dns", use, write=True)
        self.use_global_dns = use
        self.wifi.use_global_dns = use
        self.wired.use_global_dns = use

    @dbus.service.method('org.wicd.daemon')
    def SetGlobalDNS(self, dns1=None, dns2=None, dns3=None,
                     dns_dom =None, search_dom=None):
        """ Sets the global dns addresses. """
        print "setting global dns"
        self.config.set("Settings", "global_dns_1", misc.noneToString(dns1))
        self.dns1 = dns1
        self.wifi.global_dns_1 = dns1
        self.wired.global_dns_1 = dns1
        self.config.set("Settings", "global_dns_2", misc.noneToString(dns2))
        self.dns2 = dns2
        self.wifi.global_dns_2 = dns2
        self.wired.global_dns_2 = dns2
        self.config.set("Settings", "global_dns_3", misc.noneToString(dns3))
        self.dns3 = dns3
        self.wifi.global_dns_3 = dns3
        self.wired.global_dns_3 = dns3
        self.config.set("Settings", "global_dns_dom", misc.noneToString(dns_dom))
        self.dns_dom = dns_dom
        self.wifi.dns_dom = dns_dom
        self.wired.dns_dom = dns_dom
        self.config.set("Settings", "global_search_dom", misc.noneToString(search_dom))
        self.search_dom = search_dom
        self.wifi.global_search_dom = search_dom
        self.wired.global_search_dom = search_dom
        print 'global dns servers are', dns1, dns2, dns3
        print 'domain is %s' % dns_dom
        print 'search domain is %s' % search_dom
        self.config.write()

    @dbus.service.method('org.wicd.daemon')
    def SetBackend(self, backend):
        """ Sets a new backend. """
        print "setting backend to %s" % backend
        backends = networking.BACKEND_MGR.get_available_backends()
        if backend not in backends:
            print "backend %s not available, trying to fallback to another" % backend
            try:
                backend = backends[0]
            except IndexError:
                print "ERROR: no backends available!"
                return
            else:
                print "Fell back to backend %s" % backend
        self.config.set("Settings", "backend", backend, write=True)
        if backend != self.GetCurrentBackend():
            self.suspended = True
            self.wifi.LoadBackend(backend)
            self.wired.LoadBackend(backend)
            self.SignalBackendChanged(self.GetBackendUpdateInterval())
            self.SetSuspend(False)

    @dbus.service.method('org.wicd.daemon')
    def GetCurrentBackend(self):
        """ Returns the currently loaded backend. """
        return networking.get_current_backend()

    @dbus.service.method('org.wicd.daemon')
    def GetBackendUpdateInterval(self):
        """ Returns status update interval for the loaded backend. """
        return networking.get_backend_update_interval()

    @dbus.service.method('org.wicd.daemon')
    def GetBackendDescription(self, backend_name):
        """ Returns the description of the given backend. """
        return networking.get_backend_description(backend_name)

    @dbus.service.method('org.wicd.daemon')
    def GetBackendDescriptionDict(self):
        """ Returns a dict of all backend names mapped to their description. """
        return networking.get_backend_description_dict()

    @dbus.service.method('org.wicd.daemon')
    def GetSavedBackend(self):
        """ Returns the backend saved to disk. """
        return self.config.get("Settings", "backend")

    @dbus.service.method('org.wicd.daemon')
    def GetBackendList(self):
        """ Returns a list of all backends available. """
        return networking.get_backend_list()

    @dbus.service.method('org.wicd.daemon')
    def GetUseGlobalDNS(self):
        """ Returns a boolean that determines if global dns is enabled. """
        return bool(self.use_global_dns)

    @dbus.service.method('org.wicd.daemon')
    def GetWPADriver(self):
        """ Returns the wpa driver the daemon is using. """
        return str(self.wifi.wpa_driver)

    @dbus.service.method('org.wicd.daemon')
    def GetWiredInterface(self):
        """ Returns the wired interface. """
        return str(self.wired.wired_interface)

    @dbus.service.method('org.wicd.daemon')
    def GetWirelessInterface(self):
        """ Returns the wireless interface the daemon is using. """
        return str(self.wifi.wireless_interface)

    @dbus.service.method('org.wicd.daemon')
    def NeedsExternalCalls(self):
        """ Returns true if the loaded backend needs external calls. """
        if self.wifi:
            return self.wifi.NeedsExternalCalls()
        elif self.wired:
            return self.wired.NeedsExternalCalls()
        else:
            return True

    @dbus.service.method('org.wicd.daemon')
    def SetDebugMode(self, debug):
        """ Sets if debugging mode is on or off. """
        self.config.set("Settings", "debug_mode", debug, write=True)
        self.debug_mode = misc.to_bool(debug)
        self.wifi.debug = debug
        self.wired.debug = debug
        self.wireless_bus.debug_mode = debug
        self.wired_bus.debug_mode = debug

    @dbus.service.method('org.wicd.daemon')
    def GetDebugMode(self):
        """ Returns whether debugging is enabled. """
        return bool(self.debug_mode)

    @dbus.service.method('org.wicd.daemon')
    def Disconnect(self):
        """ Disconnects all networks. """
        self.SetForcedDisconnect(True)
        self.wifi.Disconnect()
        self.wired.Disconnect()

    @dbus.service.method('org.wicd.daemon')
    def FormatSignalForPrinting(self, signal):
        """ Returns the suffix to display after the signal strength number. """
        if self.GetSignalDisplayType() == 1:
            return '%s dBm' % signal
        else:
            try:
                if int(signal) == 101:
                    return '??%'
                else:
                    return '%s%%' % signal
            except ValueError:
                return '%s%%' % signal

    @dbus.service.method('org.wicd.daemon')
    def SetSuspend(self, val):
        """ Toggles whether or not monitoring connection status is suspended """
        self.suspended = val
        if self.suspended:
            self.Disconnect()
        else:
            self.SetForcedDisconnect(False)

    @dbus.service.method('org.wicd.daemon')
    def GetSuspend(self):
        """ Returns True if the computer is in the suspend state. """
        return self.suspended

    @dbus.service.method('org.wicd.daemon')
    def AutoConnect(self, fresh):
        """ Attempts to autoconnect to a wired or wireless network.

        Autoconnect will first try to connect to a wired network, if that 
        fails it tries a wireless connection.

        """
        print "Autoconnecting..."
        if self.CheckIfConnecting():
            if self.debug_mode:
                print 'Already connecting, doing nothing.'
            return
        # We don't want to rescan/connect if the gui is open.
        if self.gui_open:
            if self.debug_mode:
                print "Skipping autoconnect because GUI is open."
            return
        if self.wired_bus.CheckPluggedIn():
            if self.debug_mode:
                print "Starting wired autoconnect..."
            self._wired_autoconnect(fresh)
        else:
            if self.debug_mode:
                print "Starting wireless autoconnect..."
            self.wireless_bus._wireless_autoconnect(fresh)

    @dbus.service.method('org.wicd.daemon')
    def GetAutoReconnect(self):
        """ Returns the value of self.auto_reconnect. See SetAutoReconnect. """
        return bool(self.auto_reconnect)

    @dbus.service.method('org.wicd.daemon')
    def SetAutoReconnect(self, value):
        """ Sets the value of self.auto_reconnect.

        If True, wicd will try to reconnect as soon as it detects that
        an internet connection is lost.  If False, it will do nothing,
        and wait for the user to initiate reconnection.

        """
        print 'setting automatically reconnect when connection drops %s' % value
        self.config.set("Settings", "auto_reconnect", misc.to_bool(value), 
                        write=True)
        self.auto_reconnect = misc.to_bool(value)

    @dbus.service.method('org.wicd.daemon')
    def GetGlobalDNSAddresses(self):
        """ Returns the global dns addresses. """
        return (misc.noneToString(self.dns1), misc.noneToString(self.dns2),
                misc.noneToString(self.dns3), misc.noneToString(self.dns_dom),
                misc.noneToString(self.search_dom))

    @dbus.service.method('org.wicd.daemon')
    def CheckIfConnecting(self):
        """ Returns if a network connection is being made. """
        if self.wired_bus.CheckIfWiredConnecting() or \
           self.wireless_bus.CheckIfWirelessConnecting():
            return True
        else:
            return False

    @dbus.service.method('org.wicd.daemon')
    def CancelConnect(self):
        """ Cancels the wireless connection attempt """
        print 'canceling connection attempt'
        if self.wifi.connecting_thread:
            self.wifi.connecting_thread.should_die = True
            self.wifi.ReleaseDHCP()
            # We have to actually kill dhcp if its still hanging
            # around.  It could still be trying to get a lease.
            self.wifi.KillDHCP()
            self.wifi.StopWPA()
            self.wifi.connecting_thread.connect_result = 'aborted'
        if self.wired.connecting_thread:
            self.wired.connecting_thread.should_die = True
            self.wired.ReleaseDHCP()
            self.wired.KillDHCP()
            self.wired.connecting_thread.connect_result = 'aborted'

    @dbus.service.method('org.wicd.daemon')
    def GetCurrentInterface(self):
        """ Returns the active interface """
        return self.current_interface

    @dbus.service.method('org.wicd.daemon')    
    def SetCurrentInterface(self, iface):
        """ Sets the current active interface """
        self.current_interface = str(iface)

    @dbus.service.method('org.wicd.daemon')
    def SetNeedWiredProfileChooser(self, val):
        """ Sets the need_wired_profile_chooser variable.

        If set to True, that alerts the wicd frontend to display the chooser,
        if False the frontend will do nothing.  This function is only needed
        when the frontend starts up, to determine if the chooser was requested
        before the frontend was launched.

        """
        self.need_profile_chooser = misc.to_bool(val)

    @dbus.service.method('org.wicd.daemon')
    def ShouldAutoReconnect(self):
        """ Returns True if it's the right time to try autoreconnecting. """
        if self.GetAutoReconnect() and not self.CheckIfConnecting() and \
           not self.GetForcedDisconnect() and not self.auto_connecting and \
           not self.gui_open:
            return True
        else:
            return False

    @dbus.service.method('org.wicd.daemon')
    def GetForcedDisconnect(self):
        """ Returns the forced_disconnect status.  See SetForcedDisconnect. """
        return bool(self.forced_disconnect)

    @dbus.service.method('org.wicd.daemon')
    def SetForcedDisconnect(self, value):
        """ Sets the forced_disconnect status.

        Set to True when a user manually disconnects or cancels a connection.
        It gets set to False as soon as the connection process is manually
        started.

        """
        if self.debug_mode and value: print "Forced disconnect on"
        self.forced_disconnect = bool(value)

    @dbus.service.method('org.wicd.daemon')
    def GetSignalDisplayType(self):
        """ Returns the signal display type.

        Returns either 0 or 1.
        0 for signal strength as a percentage
        1 for signal strength measured in dBm

        """
        return int(self.signal_display_type)

    @dbus.service.method('org.wicd.daemon')
    def SetSignalDisplayType(self, value):
        """ Sets the signal display type and writes it the wicd config file. """
        self.config.set("Settings", "signal_display_type", value, write=True)
        self.signal_display_type = int(value)

    @dbus.service.method('org.wicd.daemon')
    def GetGUIOpen(self):
        """ Returns the value of gui_open.

        Returns the vlaue of gui_open, which is a boolean that keeps track
        of the state of the wicd GUI.  If the GUI is open, wicd will not
        try to automatically reconnect to networks, as this behavior can
        be annoying for the user while trying to use the GUI.

        NOTE: It's possible for this to become out of sync, particularly if
        the wicd.py is not exited properly while the GUI is open.  We should
        probably implement some kind of pid system to do it properly.

        ANOTHER NOTE: This isn't used by anything yet!

        """
        return bool(self.gui_open)

    @dbus.service.method('org.wicd.daemon')
    def SetGUIOpen(self, val):
        """ Sets the value of gui_open. """
        self.gui_open = bool(val)

    @dbus.service.method('org.wicd.daemon')
    def SetAlwaysShowWiredInterface(self, value):
        """ Sets always_show_wired_interface to the given value. """
        self.config.set("Settings", "always_show_wired_interface", 
                        misc.to_bool(value), write=True)
        self.always_show_wired_interface = misc.to_bool(value)

    @dbus.service.method('org.wicd.daemon')
    def GetAlwaysShowWiredInterface(self):
        """ Returns always_show_wired_interface """
        return bool(self.always_show_wired_interface)

    @dbus.service.method('org.wicd.daemon')
    def SetWiredAutoConnectMethod(self, method):
        """ Sets which method to use to autoconnect to wired networks. """
        # 1 = default profile
        # 2 = show list
        # 3 = last used profile
        self.config.set("Settings", "wired_connect_mode", int(method),
                        write=True)
        self.wired_connect_mode = int(method)
        self.wired_bus.connect_mode = int(method)
        
    @dbus.service.method('org.wicd.daemon')
    def SetShouldVerifyAp(self, value):
        """ Enable/disable wireless AP verification.
        
        If this is True, wicd will try to verify that we are associated
        with the Wireless AP after a connection attempt appears to
        succeed.
        
        """
        self.config.set("Settings", "should_verify_ap", int(value), write=True)
        self.wifi.should_verify_ap = misc.to_bool(value)
        
    @dbus.service.method('org.wicd.daemon')
    def GetShouldVerifyAp(self):
        """ Returns current value for WAP connection verification. """
        return bool(self.wifi.should_verify_ap)

    @dbus.service.method('org.wicd.daemon')
    def GetWiredAutoConnectMethod(self):
        """ Returns the wired autoconnect method. """
        return int(self.wired_connect_mode)

    @dbus.service.method('org.wicd.daemon')
    def GetPreferWiredNetwork(self):
        """ Returns True if wired network preference is set. 

        If this is True, wicd will switch from a wireless connection
        to a wired one if an ethernet connection is available.

        """
        return self.prefer_wired

    @dbus.service.method('org.wicd.daemon')
    def SetPreferWiredNetwork(self, value):
        """ Sets the prefer_wired state. """
        self.config.set("Settings", "prefer_wired", bool(value), write=True)
        self.prefer_wired = bool(value)

    @dbus.service.method('org.wicd.daemon')
    def GetShowNeverConnect(self):
        """ Returns True if show_never_connect is set

        if True then the client will show networks marked as never connect
        """
        return self.show_never_connect

    @dbus.service.method('org.wicd.daemon')
    def SetShowNeverConnect(self, value):
        """ Sets the how_never_connect state. """
        self.config.set("Settings", "show_never_connect", bool(value), write=True)
        self.show_never_connect = bool(value)

    @dbus.service.method('org.wicd.daemon')
    def SetConnectionStatus(self, state, info):
        """ Sets the connection status.

        Keyword arguments:
        state - An int representing the state of the connection as defined
        in misc.py.

        info - a list of strings containing data about the connection state.  
        The contents of this list are dependent on the connection state.

        state - info contents:
        NOT_CONNECTED - info[0] = ""
        CONNECTING - info[0] = "wired" or "wireless"
                     info[1] = None if wired, an essid if wireless
        WIRED - info[0] = IP Adresss
        WIRELESS - info[0] = IP Address
                   info[1] = essid
                   info[2] = signal strength
                   info[3] = internal networkid
                   info[4] = bitrate
        SUSPENDED - info[0] = ""


        """
        self.connection_state = state
        self.connection_info = info

    @dbus.service.method('org.wicd.daemon', out_signature='(uas)')
    def GetConnectionStatus(self):
        """ Returns the current connection state in list form. 

        See SetConnectionStatus for more information about the
        data structure being returned.

        """
        return [self.connection_state, self.connection_info]

    @dbus.service.method('org.wicd.daemon')
    def GetNeedWiredProfileChooser(self):
        """ Returns need_profile_chooser.

        Returns a boolean specifying if the wired profile chooser needs to
        be launched.

        """
        return bool(self.need_profile_chooser)

    @dbus.service.method("org.wicd.daemon")
    def GetAppAvailable(self, app):
        """ Determine if a given  application is available."""
        return bool(self.wifi.AppAvailable(app) or self.wired.AppAvailable(app))

    @dbus.service.method('org.wicd.daemon')
    def GetDHCPClient(self):
        """ Returns the current DHCP client constant.

        See misc.py for a definition of the constants.

        """
        return self.dhcp_client

    @dbus.service.method('org.wicd.daemon')
    def SetDHCPClient(self, client):
        """ Sets the DHCP client constant.

        See misc.py for a definition of the constants.

        """
        print "Setting dhcp client to %i" % (int(client))
        self.dhcp_client = int(client)
        self.wifi.dhcp_client = int(client)
        self.wired.dhcp_client = int(client)
        self.config.set("Settings", "dhcp_client", client, write=True)

    @dbus.service.method('org.wicd.daemon')
    def GetLinkDetectionTool(self):
        """ Returns the current link detection tool constant. """
        return self.link_detect_tool

    @dbus.service.method('org.wicd.daemon')
    def SetLinkDetectionTool(self, link_tool):
        """ Sets the link detection tool. 

        Sets the value of the tool wicd should use to detect if a
        cable is plugged in.  If using a backend that doesn't use
        an external call to get this information (such as ioctl)
        it will instead use the ioctls provided by the specified
        tool to query for link status.

        """
        self.link_detect_tool = int(link_tool)
        self.wired.link_detect = int(link_tool)
        self.config.set("Settings", "link_detect_tool", link_tool, write=True)

    @dbus.service.method('org.wicd.daemon')
    def GetFlushTool(self):
        """ Returns the current flush tool constant. """
        return self.flush_tool

    @dbus.service.method('org.wicd.daemon')
    def SetFlushTool(self, flush_tool):
        """ Sets the flush tool.

        Sets the value of the tool wicd should use to flush routing tables.
        The value is associated with a particular tool, as specified in
        misc.py

        """
        self.flush_tool = int(flush_tool)
        self.wired.flush_tool = int(flush_tool)
        self.wifi.flush_tool = int(flush_tool)
        self.config.set("Settings", "flush_tool", flush_tool, write=True)

    @dbus.service.method('org.wicd.daemon')
    def GetSudoApp(self):
        """ Get the preferred sudo app. """
        return self.sudo_app

    @dbus.service.method('org.wicd.daemon')
    def SetSudoApp(self, sudo_app):
        """ Set the preferred sudo app. """
        self.sudo_app = sudo_app
        self.config.set("Settings", "sudo_app", sudo_app, write=True)

    def _wired_autoconnect(self, fresh=True):
        """ Attempts to autoconnect to a wired network. """
        wiredb = self.wired_bus
        if self.GetWiredAutoConnectMethod() == 3 and \
           not self.GetNeedWiredProfileChooser():
            # attempt to smartly connect to a wired network
            # by using various wireless networks detected
            # and by using plugged in USB devices
            print self.LastScan
        if self.GetWiredAutoConnectMethod() == 2 and \
           not self.GetNeedWiredProfileChooser():
            self.LaunchChooser()
            return True

        # Default Profile.
        elif self.GetWiredAutoConnectMethod() == 1:
            network = wiredb.GetDefaultWiredNetwork()
            if not network:
                print "Couldn't find a default wired connection," + \
                      " wired autoconnect failed."
                self.wireless_bus._wireless_autoconnect(fresh)
                return

        # Last-Used.
        else:
            network = wiredb.GetLastUsedWiredNetwork()
            if not network:
                print "no previous wired profile available, wired " + \
                      "autoconnect failed."
                self.wireless_bus._wireless_autoconnect(fresh)
                return

        wiredb.ReadWiredNetworkProfile(network)
        wiredb.ConnectWired()
        print "Attempting to autoconnect with wired interface..."
        self.auto_connecting = True
        time.sleep(1.5)
        try:
            gobject.timeout_add_seconds(3, self._monitor_wired_autoconnect, 
                                        fresh)
        except:
            gobject.timeout_add(3000, self._monitor_wired_autoconnect, fresh)
        return True

    def _monitor_wired_autoconnect(self, fresh):
        """ Monitor a wired auto-connection attempt.

        Helper method called on a timer that monitors a wired
        connection attempt and makes decisions about what to
        do next based on the result.

        """
        wiredb = self.wired_bus
        if wiredb.CheckIfWiredConnecting():
            return True
        elif wiredb.GetWiredIP():
            self.auto_connecting = False
            return False
        elif not self.wireless_bus.CheckIfWirelessConnecting():
            self.wireless_bus._wireless_autoconnect(fresh)
            return False
        self.auto_connecting = False
        return False

    @dbus.service.method("org.wicd.daemon")
    def ConnectResultsAvailable(self):
        if ((self.wired.connecting_thread and self.wired.connecting_thread.connect_result) or 
            (self.wifi.connecting_thread and self.wifi.connecting_thread.connect_result)):
            return True
        else:
            return False

    @dbus.service.method("org.wicd.daemon")
    def SendConnectResultsIfAvail(self):
        if self.ConnectResultsAvailable():
            self.SendConnectResult()

    @dbus.service.method("org.wicd.daemon")
    def SendConnectResult(self):
        if self.wired.connecting_thread and self.wired.connecting_thread.connect_result:
            self.ConnectResultsSent(self.wired.connecting_thread.connect_result)
            self.wired.connecting_thread.connect_result = ""
        elif self.wifi.connecting_thread and self.wifi.connecting_thread.connect_result:
            self.ConnectResultsSent(self.wifi.connecting_thread.connect_result)
            self.wifi.connecting_thread.connect_result = ""

    @dbus.service.signal(dbus_interface="org.wicd.daemon",signature='s')
    def ConnectResultsSent(self, result):
        print "Sending connection attempt result %s" % result

    @dbus.service.method("org.wicd.daemon")
    @dbus.service.signal(dbus_interface="org.wicd.daemon", signature='')
    def UpdateState(self):
        pass

    @dbus.service.signal(dbus_interface='org.wicd.daemon', signature='')
    def LaunchChooser(self):
        """ Emits the wired profile chooser dbus signal. """
        print 'calling wired profile chooser'
        self.SetNeedWiredProfileChooser(True)

    @dbus.service.signal(dbus_interface="org.wicd.daemon", signature='')
    def DaemonStarting(self):
        """ Emits a signa indicating the daemon is starting. """
        pass

    @dbus.service.signal(dbus_interface='org.wicd.daemon', signature='')
    def DaemonClosing(self):
        """ Emits a signal indicating the daemon will be closing. """
        pass

    @dbus.service.method('org.wicd.daemon', in_signature='uav')
    def EmitStatusChanged(self, state, info):
        """ Calls the StatusChanged signal method. """
        self.StatusChanged(state, info)

    @dbus.service.signal(dbus_interface='org.wicd.daemon', signature='uav')
    def StatusChanged(self, state, info):
        """ Emits a "status changed" dbus signal.

        This D-Bus signal is emitted when the connection status changes.
        This signal can be hooked to monitor the network state.

        """
        pass
    
    @dbus.service.signal(dbus_interface='org.wicd.daemon', signature='i')
    def SignalBackendChanged(self, interval):
        """ Emits a signal when the current backend changes. """
        pass

    def ReadConfig(self):
        """ Reads the manager-settings.conf file.

        Reads the manager-settings.conf file and loads the stored
        values into memory.

        """
        b_wired = self.wired_bus
        b_wifi = self.wireless_bus
        app_conf = self.config
        # Load the backend.
        be_def = 'external'
        self.SetBackend(app_conf.get("Settings", "backend", default=be_def))

        # Load network interfaces.
        iface = self.wireless_bus.DetectWirelessInterface()
        if not iface: iface = 'wlan0'
        self.SetWirelessInterface(app_conf.get("Settings", "wireless_interface",
                                               default=iface))
        iface = self.wired_bus.DetectWiredInterface()
        if not iface: iface = 'eth0'
        self.SetWiredInterface(app_conf.get("Settings", "wired_interface",
                                            default=iface))

        self.SetWPADriver(app_conf.get("Settings", "wpa_driver", default="wext"))
        self.SetAlwaysShowWiredInterface(app_conf.get("Settings",
                                                      "always_show_wired_interface",
                                                      default=False))
        self.SetUseGlobalDNS(app_conf.get("Settings", "use_global_dns",
                                          default=False))
        dns1 = app_conf.get("Settings", "global_dns_1", default='None')
        dns2 = app_conf.get("Settings", "global_dns_2", default='None')
        dns3 = app_conf.get("Settings", "global_dns_3", default='None')
        dns_dom = app_conf.get("Settings", "global_dns_dom", default='None')
        search_dom = app_conf.get("Settings", "global_search_dom", default='None')
        self.SetGlobalDNS(dns1, dns2, dns3, dns_dom, search_dom)
        self.SetAutoReconnect(app_conf.get("Settings", "auto_reconnect",
                                           default=True))
        self.SetDebugMode(app_conf.get("Settings", "debug_mode", default=False))
        self.SetWiredAutoConnectMethod(app_conf.get("Settings",
                                                    "wired_connect_mode",
                                                    default=1))
        self.SetSignalDisplayType(app_conf.get("Settings", 
                                               "signal_display_type",
                                               default=0))
        self.SetShouldVerifyAp(app_conf.get("Settings", "should_verify_ap",
                                            default=1))
        self.SetDHCPClient(app_conf.get("Settings", "dhcp_client", default=0))
        self.SetLinkDetectionTool(app_conf.get("Settings", "link_detect_tool",
                                               default=0))
        self.SetFlushTool(app_conf.get("Settings", "flush_tool", default=0))
        self.SetSudoApp(app_conf.get("Settings", "sudo_app", default=0))
        self.SetPreferWiredNetwork(app_conf.get("Settings", "prefer_wired", 
                                                default=False))
        self.SetShowNeverConnect(app_conf.get("Settings", "show_never_connect", 
                                                default=True))
        app_conf.write()

        if os.path.isfile(wireless_conf):
            print "Wireless configuration file found..."
        else:
            print "Wireless configuration file not found, creating..."
            open(wireless_conf, "w").close()

        if os.path.isfile(wired_conf):
            print "Wired configuration file found..."
        else:
            print "Wired configuration file not found, creating a default..."
            # Create the file and a default profile
            open(wired_conf, "w").close()
            b_wired.CreateWiredNetworkProfile("wired-default", default=True)

        if not os.path.isfile(dhclient_conf):
            print "dhclient.conf.template not found, copying..."
            shutil.copy(dhclient_conf + ".default", dhclient_conf)            
        # Hide the files, so the keys aren't exposed.
        print "chmoding configuration files 0600..."
        os.chmod(app_conf.get_config(), 0600)
        os.chmod(wireless_conf, 0600)
        os.chmod(wired_conf, 0600)
        os.chmod(dhclient_conf, 0644)

        # Make root own them
        print "chowning configuration files root:root..."
        os.chown(app_conf.get_config(), 0, 0)
        os.chown(wireless_conf, 0, 0)
        os.chown(wired_conf, 0, 0)
        os.chown(dhclient_conf, 0, 0)

        print "Using wireless interface..." + self.GetWirelessInterface()
        print "Using wired interface..." + self.GetWiredInterface()

##############################
###### Wireless Daemon #######
##############################

class WirelessDaemon(dbus.service.Object):
    """ DBus interface for wireless connection operations. """
    def __init__(self, bus_name, daemon, wifi=None, debug=False):
        """ Intitialize the wireless DBus interface. """
        dbus.service.Object.__init__(self, bus_name=bus_name,
                                     object_path='/org/wicd/daemon/wireless')
        self.hidden_essid = None
        self.daemon = daemon
        self.wifi = wifi
        self._debug_mode = debug
        self._scanning = False
        self.LastScan = []
        self.config = ConfigManager(wireless_conf, debug=debug)

        self._validProperties = (
            'bssid',
            'essid',
            'hidden',
            'channel',
            'mode',
            'enctype',
            'encryption_method',
            'key',
            'automatic',
            'ip',
            'netmask',
            'broadcast',
            'gateway',
            'use_static_dns',
            'use_global_dns',
            'dns1',
            'dns2',
            'dns3',
            'use_settings_globally',
        )

    def get_debug_mode(self):
        return self._debug_mode
    def set_debug_mode(self, mode):
        self._debug_mode = mode
        self.config.debug = mode
    debug_mode = property(get_debug_mode, set_debug_mode)

    @dbus.service.method('org.wicd.daemon.wireless')
    def SetHiddenNetworkESSID(self, essid):
        """ Sets the ESSID of a hidden network for use with Scan(). """
        self.hidden_essid = str(misc.Noneify(essid))

    @dbus.service.method('org.wicd.daemon.wireless')
    def Scan(self, sync=False):
        """ Scan for wireless networks.

        Scans for wireless networks, optionally using a (hidden) essid
        set with SetHiddenNetworkESSID.

        The sync keyword argument specifies whether the scan should
        be done synchronously.

        """
        if self._scanning:
            if self.debug_mode:
                print "scan already in progress, skipping"
            return False
        if self.debug_mode:
            print 'scanning start'
        self.SendStartScanSignal()
        if sync:
            self._sync_scan()
        else:
            self._async_scan()
        return True

    @misc.threaded
    def _async_scan(self):
        """ Run a scan in its own thread. """
        self._sync_scan()

    def _sync_scan(self):
        """ Run a scan and send a signal when its finished. """
        scan = self.wifi.Scan(str(self.hidden_essid))
        self.LastScan = scan
        if self.debug_mode:
            print 'scanning done'
            print 'found ' + str(len(scan)) + ' networks:'
        for i, network in enumerate(scan):
            self.ReadWirelessNetworkProfile(i)
        self.SendEndScanSignal()

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetIwconfig(self):
        """ Calls and returns the output of iwconfig"""
        return misc.to_unicode(self.wifi.GetIwconfig())

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetNumberOfNetworks(self):
        """ Returns number of networks. """
        return len(self.LastScan)

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetApBssid(self):
        """ Gets the MAC address for the active network. """
        return self.wifi.GetBSSID()

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetCurrentBitrate(self, iwconfig):
        """ Returns the current bitrate for the active network. """
        return self.wifi.GetCurrentBitrate(iwconfig)

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetOperationalMode(self, iwconfig):
        """ Returns the operational mode for the iwconfig parameter """
        return misc.to_unicode(self.wifi.GetOperationalMode(iwconfig))

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetAvailableAuthMethods(self, iwlistauth):
        """ Returns the operational mode for the iwlistauth parameter """
        return misc.to_unicode(self.wifi.GetAvailableAuthMethods(iwlistauth))

    @dbus.service.method('org.wicd.daemon.wireless')
    def CreateAdHocNetwork(self, essid, channel, ip, enctype, key, encused,
                           ics):
        """ Creates an ad-hoc network using user inputted settings. """
        self.wifi.CreateAdHocNetwork(essid, channel, ip, enctype, key, encused)

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetKillSwitchEnabled(self):
        """ Returns true if kill switch is pressed. """
        status = self.wifi.GetKillSwitchStatus()
        return status

    @dbus.service.method('org.wicd.daemon.wireless')
    def SwitchRfKill(self):
        """ Switches the rfkill on/off for wireless cards. """
        return self.wifi.SwitchRfKill()

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetRfKillEnabled(self):
        """ Returns true if rfkill switch is enabled. """
        return self.wifi.GetRfKillStatus()

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetWirelessProperty(self, networkid, property):
        """ Retrieves wireless property from the network specified """
        try:
            value = self.LastScan[networkid].get(property)
        except IndexError:
            return ""
        value = misc.to_unicode(value)
        return value

    @dbus.service.method('org.wicd.daemon.wireless')
    def SetWirelessProperty(self, netid, prop, value):
        """ Sets property to value in network specified. """
        # We don't write script settings here.
        if prop.strip() not in self._validProperties:
            print "Trying to set invalid property (or property not " \
                  "permitted): "+ prop.strip() + "."
            return False
        self.LastScan[netid][prop] = misc.to_unicode(misc.Noneify(value))

    @dbus.service.method('org.wicd.daemon.wireless')
    def DetectWirelessInterface(self):
        """ Returns an automatically detected wireless interface. """
        iface = self.wifi.DetectWirelessInterface()
        if iface:
            print 'Automatically detected wireless interface ' + iface
        else:
            print "Couldn't detect a wireless interface."
        return str(iface)

    @dbus.service.method('org.wicd.daemon.wireless')
    def DisconnectWireless(self):
        """ Disconnects the wireless network. """
        self.wifi.Disconnect()
        self.daemon.UpdateState()

    @dbus.service.method('org.wicd.daemon.wireless')
    def IsWirelessUp(self):
        """ Returns a boolean specifying if wireless is up or down. """
        return self.wifi.IsUp()

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetCurrentSignalStrength(self, iwconfig=None):
        """ Returns the current signal strength. """
        try:
            strength = int(self.wifi.GetSignalStrength(iwconfig))
        except TypeError:
            strength = 0
        return strength

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetCurrentDBMStrength(self, iwconfig=None):
        """ Returns the current dbm signal strength. """
        try:
            dbm_strength = int(self.wifi.GetDBMStrength(iwconfig))
        except:
            dbm_strength = 0
        return dbm_strength

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetCurrentNetwork(self, iwconfig=None):
        """ Returns the current network. """
        current_network = str(self.wifi.GetCurrentNetwork(iwconfig))
        return current_network

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetCurrentNetworkID(self, iwconfig=None):
        """ Returns the id of the current network, or -1 if its not found. """
        currentESSID = self.GetCurrentNetwork(iwconfig)
        for x in xrange(0, len(self.LastScan)):
            if self.LastScan[x]['essid'] == currentESSID:
                return x
        if self.debug_mode:
            print 'GetCurrentNetworkID: Returning -1, current network not found'
        return -1

    @dbus.service.method('org.wicd.daemon.wireless')
    def EnableWirelessInterface(self):
        """ Calls a method to enable the wireless interface. """
        result = self.wifi.EnableInterface()
        return result

    @dbus.service.method('org.wicd.daemon.wireless')
    def DisableWirelessInterface(self):
        """ Calls a method to disable the wireless interface. """
        result = self.wifi.DisableInterface()
        return result

    @dbus.service.method('org.wicd.daemon.wireless')
    def ConnectWireless(self, id):
        """ Connects the the wireless network specified by i"""
        self.SaveWirelessNetworkProfile(id)
        # Will returned instantly, that way we don't hold up dbus.
        # CheckIfWirelessConnecting can be used to test if the connection
        # is done.
        self.wifi.before_script = self.GetWirelessProperty(id, 'beforescript')
        self.wifi.after_script = self.GetWirelessProperty(id, 'afterscript')
        self.wifi.pre_disconnect_script = self.GetWirelessProperty(id,
                                                               'predisconnectscript')
        self.wifi.post_disconnect_script = self.GetWirelessProperty(id,
                                                               'postdisconnectscript')
        print 'Connecting to wireless network ' + str(self.LastScan[id]['essid'])
        # disconnect to make sure that scripts are run
        self.wifi.Disconnect()
        self.daemon.wired_bus.wired.Disconnect()
        self.daemon.SetForcedDisconnect(False)
        conthread = self.wifi.Connect(self.LastScan[id], debug=self.debug_mode)
        self.daemon.UpdateState()

    @dbus.service.method('org.wicd.daemon.wireless')
    def CheckIfWirelessConnecting(self):
        """Returns True if wireless interface is connecting, otherwise False."""
        if self.wifi.connecting_thread:
            return self.wifi.connecting_thread.is_connecting
        else:
            return False

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetWirelessIP(self, ifconfig=""):
        """ Returns the IP associated with the wireless interface. """
        ip = self.wifi.GetIP(ifconfig)
        return ip

    @dbus.service.method('org.wicd.daemon.wireless')
    def CheckWirelessConnectingStatus(self):
        """ Returns the wireless interface's status code. """
        if self.wifi.connecting_thread:
            stat = self.wifi.connecting_thread.GetStatus()
            return stat
        else:
            return False

    @dbus.service.method('org.wicd.daemon.wireless')
    def CheckWirelessConnectingMessage(self):
        """ Returns the wireless interface's status message. """
        if self.wifi.connecting_thread:
            stat = self.CheckWirelessConnectingStatus()
            return _status_dict[stat]
        else:
            return False

    @dbus.service.method('org.wicd.daemon.wireless')
    def ReadWirelessNetworkProfile(self, id):
        """ Reads in wireless profile as the active network """
        cur_network = self.LastScan[id]
        essid_key = "essid:%s" % cur_network["essid"]
        bssid_key = cur_network["bssid"]

        if self.config.get(essid_key, 'use_settings_globally'):
            section = essid_key
        elif self.config.has_section(bssid_key):
            section = bssid_key
        else:
            return "500: Profile Not Found"

        for x in self.config.options(section):
            if not cur_network.has_key(x) or x.endswith("script"):
                cur_network[x] = misc.Noneify(self.config.get(section, x))
        for option in ['use_static_dns', 'use_global_dns', 'encryption',
                       'use_settings_globally']:
            cur_network[option] = bool(cur_network.get(option))
        # Read the essid because we need to name those hidden
        # wireless networks now - but only read it if it is hidden.
        if cur_network["hidden"]:
            # check if there is an essid in the config file
            # if there isn't, .get( will return None
            stored_essid = self.config.get(section, 'essid')
            if stored_essid:
                # set the current network's ESSID to the stored one
                cur_network['essid'] = stored_essid

    @dbus.service.method('org.wicd.daemon.wireless')
    def SaveWirelessNetworkProfile(self, id):
        """ Writes a wireless profile to disk. """
        def write_script_ent(prof, script):
            if not self.config.has_option(prof, script):
                self.config.set(prof, script, None)

        cur_network = self.LastScan[id]
        bssid_key = cur_network["bssid"]
        essid_key = "essid:%s" % cur_network["essid"]

        self.config.remove_section(bssid_key)
        self.config.add_section(bssid_key)

        # We want to write the essid in addition to bssid
        # sections if global settings are enabled.
        self.config.remove_section(essid_key)
        if cur_network.get("use_settings_globally", False):
            self.config.add_section(essid_key)

        for x in cur_network:
            # There's no reason to save these to a configfile...
            if x not in ['quality', 'strength', 'bitrates', 'has_profile']:
                self.config.set(bssid_key, x, cur_network[x])
                if cur_network.get("use_settings_globally", False):
                    self.config.set(essid_key, x, cur_network[x])

        write_script_ent(bssid_key, "beforescript")
        write_script_ent(bssid_key, "afterscript")
        write_script_ent(bssid_key, "predisconnectscript")
        write_script_ent(bssid_key, "postdisconnectscript")

        if cur_network.get("use_settings_globally", False):
            write_script_ent(essid_key, "beforescript")
            write_script_ent(essid_key, "afterscript")
            write_script_ent(essid_key, "predisconnectscript")
            write_script_ent(essid_key, "postdisconnectscript")

        self.config.write()

    @dbus.service.method('org.wicd.daemon.wireless')
    def SaveWirelessNetworkProperty(self, id, option):
        """ Writes a particular wireless property to disk. """
        if (option.strip()).endswith("script"):
            print 'You cannot save script information to disk through ' + \
                  'the daemon.'
            return
        config = self.config
        cur_network = self.LastScan[id]
        essid_key = "essid:" + cur_network["essid"]

        config.set(cur_network["bssid"], option, str(cur_network[option]))

        # Write the global section as well, if required.
        if config.get(essid_key, 'use_settings_globally'):
            config.set(essid_key, option, str(cur_network[option]))
        config.write()

    @dbus.service.method('org.wicd.daemon.wireless')
    def RemoveGlobalEssidEntry(self, networkid):
        """ Removes the global entry for the networkid provided. """
        essid_key = "essid:" + str(self.LastScan[networkid])
        self.config.remove_section(essid_key)

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetWpaSupplicantDrivers(self):
        """ Returns all valid wpa_supplicant drivers. """
        return self.wifi.GetWpaSupplicantDrivers()

    @dbus.service.method('org.wicd.daemon.wireless')
    def ReloadConfig(self):
        """ Reloads the active config file. """
        self.config.reload()

    @dbus.service.method('org.wicd.daemon.wireless')
    def GetWirelessInterfaces(self):
        ''' Returns a list of wireless interfaces on the system. '''
        return wnettools.GetWirelessInterfaces()

    @dbus.service.signal(dbus_interface='org.wicd.daemon.wireless', signature='')
    def SendStartScanSignal(self):
        """ Emits a signal announcing a scan has started. """
        self._scanning = True

    @dbus.service.signal(dbus_interface='org.wicd.daemon.wireless', signature='')
    def SendEndScanSignal(self):
        """ Emits a signal announcing a scan has finished. """
        self._scanning = False

    def _wireless_autoconnect(self, fresh=True):
        """ Attempts to autoconnect to a wireless network. """
        print "No wired connection present, attempting to autoconnect " + \
              "to wireless network"
        if self.wifi.wireless_interface is None:
            print 'Autoconnect failed because wireless interface returned None'
            return
        if fresh:
            self.Scan(sync=True)

        for x, network in enumerate(self.LastScan):
            if self.config.has_section(network['bssid']):
                if self.debug_mode:
                    print network["essid"] + ' has profile'
                if bool(network.get('automatic')):
                    try:
                        if network.get('never'):
                            print network["essid"],'marked never connect'
                            continue
                    except:
                        print network["essid"],'has no never connect value'
                    print 'trying to automatically connect to...' + \
                          network["essid"]
                    self.ConnectWireless(x)
                    time.sleep(1)
                    return
        print "Unable to autoconnect, you'll have to manually connect"

###########################
###### Wired Daemon #######
###########################

class WiredDaemon(dbus.service.Object):
    """ DBus interface for wired connection operations. """
    def __init__(self, bus_name, daemon, wired=None, debug=False):
        """ Intitialize the wireless DBus interface. """
        dbus.service.Object.__init__(self, bus_name=bus_name,
                                     object_path="/org/wicd/daemon/wired")
        self.daemon = daemon
        self.wired = wired
        self._debug_mode = debug
        self._cur_wired_prof_name = ""
        self.WiredNetwork = {}
        self.config = ConfigManager(wired_conf, debug=debug)

        self._validProperties = (
            'ip',
            'broadcast',
            'netmask',
            'gateway',
            'search_domain',
            'dns_domain',
            'dns1',
            'dns2',
            'dns3',
            'encryption_enabled',
            'default',
            'dhcphostname',
            'lastused',
            'profilename',
            'use_global_dns',
            'use_static_dns',
        )

    def get_debug_mode(self):
        return self._debug_mode
    def set_debug_mode(self, mode):
        self._debug_mode = mode
        self.config.debug = mode
    debug_mode = property(get_debug_mode, set_debug_mode)

    @dbus.service.method('org.wicd.daemon.wired')
    def GetWiredIP(self, ifconfig=""):
        """ Returns the wired interface's ip address. """
        ip = self.wired.GetIP(ifconfig)
        return ip

    @dbus.service.method('org.wicd.daemon.wired')
    def CheckIfWiredConnecting(self):
        """ Returns True if wired interface is connecting, otherwise False. """
        if self.wired.connecting_thread:
            return self.wired.connecting_thread.is_connecting
        else:
            return False

    @dbus.service.method('org.wicd.daemon.wired')
    def CheckWiredConnectingStatus(self):
        """Returns the wired interface's status code. '"""
        if self.wired.connecting_thread:
            return self.wired.connecting_thread.GetStatus()
        else:
            return False

    @dbus.service.method('org.wicd.daemon.wired')
    def CheckWiredConnectingMessage(self):
        """ Returns the wired interface's status message. """
        if self.wired.connecting_thread:
            return _status_dict[self.CheckWiredConnectingStatus()]
        else:
            return False

    @dbus.service.method('org.wicd.daemon.wired')
    def DetectWiredInterface(self):
        """ Returns an automatically detected wireless interface. """
        iface = self.wired.DetectWiredInterface()
        if iface:
            print 'automatically detected wired interface ' + str(iface)
        else:
            print "Couldn't detect a wired interface."
        return str(iface)

    @dbus.service.method('org.wicd.daemon.wired')
    def SetWiredProperty(self, prop, value):
        """ Sets the given property to the given value. """
        if self.WiredNetwork:
            if prop.strip() not in self._validProperties:
                print "Trying to set invalid property (or property not " \
                      "permitted): "+ prop.strip() + "."
                return False
            self.WiredNetwork[prop] = misc.to_unicode(misc.Noneify(value))
            return True
        else:
            print 'SetWiredProperty: WiredNetwork does not exist'
            return False

    @dbus.service.method('org.wicd.daemon.wired')
    def GetWiredProperty(self, property):
        """ Returns the requested wired property. """
        if self.WiredNetwork:
            value = self.WiredNetwork.get(property)
            return value
        else:
            print 'GetWiredProperty: WiredNetwork does not exist'
            return False

    @dbus.service.method('org.wicd.daemon.wired')
    def HasWiredDriver(self):
        """ Returns True if a driver is associated with this interface. """
        if self.wired.driver:
            return True
        else:
            return False

    @dbus.service.method('org.wicd.daemon.wired')
    def DisconnectWired(self):
        """ Disconnects the wired network. """
        self.wired.Disconnect()
        self.daemon.UpdateState()

    @dbus.service.method('org.wicd.daemon.wired')
    def CheckPluggedIn(self):
        """ Returns True if a ethernet cable is present, False otherwise. """
        if self.wired.wired_interface and self.wired.wired_interface != "None":
            return self.wired.CheckPluggedIn()
        else:
            return None

    @dbus.service.method('org.wicd.daemon.wired')
    def IsWiredUp(self):
        """ Returns a boolean specifying if wired iface is up or down. """
        return self.wired.IsUp()

    @dbus.service.method('org.wicd.daemon.wired')
    def EnableWiredInterface(self):
        """ Calls a method to enable the wired interface. """           
        return self.wired.EnableInterface()

    @dbus.service.method('org.wicd.daemon.wired')
    def DisableWiredInterface(self):
        """ Calls a method to disable the wired interface. """
        return self.wired.DisableInterface()

    @dbus.service.method('org.wicd.daemon.wired')
    def ConnectWired(self):
        """ Connects to a wired network. """
        self.wired.before_script = self.GetWiredProperty("beforescript")
        self.wired.after_script = self.GetWiredProperty("afterscript")
        self.wired.pre_disconnect_script = self.GetWiredProperty("predisconnectscript")
        self.wired.post_disconnect_script = self.GetWiredProperty("postdisconnectscript")
        self.daemon.wireless_bus.wifi.Disconnect()
        # make sure disconnect scripts are run
        self.wired.Disconnect()
        self.daemon.SetForcedDisconnect(False)
        self.UnsetWiredLastUsed()
        self.config.set(self._cur_wired_prof_name, "lastused", True, write=True)
        self.wired.Connect(self.WiredNetwork, debug=self.debug_mode)
        self.daemon.UpdateState()

    @dbus.service.method('org.wicd.daemon.wired')
    def CreateWiredNetworkProfile(self, profilename, default=False):
        """ Creates a wired network profile. """
        if not profilename:
            return False
        profilename = misc.to_unicode(profilename)
        print "Creating wired profile for " + profilename
        if self.config.has_section(profilename):
            return False

        for option in ["ip", "broadcast", "netmask", "gateway", "search_domain", 
                       "dns_domain", "dns1", "dns2", "dns3", "beforescript", 
                       "afterscript", "predisconnectscript",
                       "postdisconnectscript", "encryption_enabled"]:
            self.config.set(profilename, option, None)
        self.config.set(profilename, "default", default)
        self.config.set(profilename,"dhcphostname",os.uname()[1])
        self.config.write()
        return True

    @dbus.service.method('org.wicd.daemon.wired')
    def UnsetWiredLastUsed(self):
        """ Finds the previous lastused network, and sets lastused to False. """
        profileList = self.config.sections()
        for profile in profileList:
            if misc.to_bool(self.config.get(profile, "lastused")):
                self.config.set(profile, "lastused", False, write=True)

    @dbus.service.method('org.wicd.daemon.wired')
    def UnsetWiredDefault(self):
        """ Unsets the default option in the current default wired profile. """
        profileList = self.config.sections()
        for profile in profileList:
            if misc.to_bool(self.config.get(profile, "default")):
                self.config.set(profile, "default", False, write=True)

    @dbus.service.method('org.wicd.daemon.wired')
    def GetDefaultWiredNetwork(self):
        """ Returns the current default wired network. """
        profileList = self.config.sections()
        for profile in profileList:
            if misc.to_bool(self.config.get(profile, "default")):
                return profile
        return None

    @dbus.service.method('org.wicd.daemon.wired')
    def GetLastUsedWiredNetwork(self):
        """ Returns the profile of the last used wired network. """
        profileList = self.config.sections()
        for profile in profileList:
            if misc.to_bool(self.config.get(profile, "lastused")):
                return profile
        return None

    @dbus.service.method('org.wicd.daemon.wired')
    def DeleteWiredNetworkProfile(self, profilename):
        """ Deletes a wired network profile. """
        profilename = misc.to_unicode(profilename)
        print "Deleting wired profile for " + str(profilename)
        self.config.remove_section(profilename)
        self.config.write()

    @dbus.service.method('org.wicd.daemon.wired')
    def SaveWiredNetworkProfile(self, profilename):
        """ Writes a wired network profile to disk. """
        def write_script_ent(prof, script):
            if not self.config.has_option(prof, script):
                self.config.set(prof, script, None)

        profilename = profilename.strip()
        if not profilename:
            self.config.write()
            print "Warning: Bad wired profile name given, ignoring."
            return "500: Bad Profile name"
        if self.debug_mode:
            print "saving wired profile %s" % profilename
        profilename = misc.to_unicode(profilename)
        self.config.remove_section(profilename)
        self.config.add_section(profilename)
        for x in self.WiredNetwork:
            self.config.set(profilename, x, self.WiredNetwork[x])

        write_script_ent(profilename, "beforescript")
        write_script_ent(profilename, "afterscript")
        write_script_ent(profilename, "predisconnectscript")
        write_script_ent(profilename, "postdisconnectscript")
        self.config.write()
        return "100: Profile Written"

    @dbus.service.method('org.wicd.daemon.wired')
    def ReadWiredNetworkProfile(self, profilename):
        """ Reads a wired network profile in as the currently active profile """
        profile = {}
        profilename = misc.to_unicode(profilename)
        if self.config.has_section(profilename):
            if self.debug_mode:
                print "Reading wired profile %s" % profilename
            for x in self.config.options(profilename):
                profile[x] = misc.Noneify(self.config.get(profilename, x))
            profile['use_global_dns'] = bool(profile.get('use_global_dns'))
            profile['use_static_dns'] = bool(profile.get('use_static_dns'))
            profile['encryption_enabled'] = bool(profile.get('encryption_enabled'))
            profile['profilename'] = profilename
            self.WiredNetwork = profile
            self._cur_wired_prof_name = profilename
            return "100: Loaded Profile"
        else:
            self._cur_wired_prof_name = ""
            self.WiredNetwork = {}
            return "500: Profile Not Found"

    @dbus.service.method('org.wicd.daemon.wired')
    def GetWiredProfileList(self):
        """ Returns a list of all wired profiles in wired-settings.conf. """
        sections = self.config.sections()
        if not sections:
            sections = [""]
        return sections

    @dbus.service.method('org.wicd.daemon.wired')
    def ReloadConfig(self):
        """ Reloads the active config file. """
        self.config.reload()

    @dbus.service.method('org.wicd.daemon.wired')
    def GetWiredInterfaces(self):
        ''' Returns a list of wireless interfaces on the system. '''
        return wnettools.GetWiredInterfaces()

def usage():
    print """
wicd %s 
wireless (and wired) connection daemon.

Arguments:
\t-a\t--no-autoconnect\tDon't auto-scan/auto-connect.
\t-f\t--no-daemon\tDon't daemonize (run in foreground).
\t-e\t--no-stderr\tDon't redirect stderr.
\t-n\t--no-poll\tDon't monitor network status.
\t-o\t--no-stdout\tDon't redirect stdout.
\t-h\t--help\t\tPrint this help.
""" % (wpath.version + ' (bzr-r%s)' % wpath.revision)

def daemonize():
    """ Disconnect from the controlling terminal.

    Fork twice, once to disconnect ourselves from the parent terminal and a
    second time to prevent any files we open from becoming our controlling
    terminal.

    For more info see:
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66012

    """
    # Fork the first time to disconnect from the parent terminal and
    # exit the parent process.
    try:
        pid = os.fork()
        if pid > 0:
            sys.exit(0)
    except OSError, e:
        print >> sys.stderr, "Fork #1 failed: %d (%s)" % (e.errno, e.strerror)
        sys.exit(1)

    # Decouple from parent environment to stop us from being a zombie.
    os.setsid()

    # Fork the second time to prevent us from opening a file that will
    # become our controlling terminal.
    try:
        pid = os.fork()
        if pid > 0:
            dirname = os.path.dirname(wpath.pidfile)
            if not os.path.exists(dirname):
                os.makedirs(dirname)
            pidfile = open(wpath.pidfile, 'w')
            pidfile.write(str(pid) + '\n')
            pidfile.close()
            sys.exit(0)
        else:
            os.umask(0)
            os.chdir('/')
    except OSError, e:
        print >> sys.stderr, "Fork #2 failed: %d (%s)" % (e.errno, e.strerror)
        sys.exit(1)

    sys.stdin.close()
    sys.stdout.close()
    sys.stderr.close()

    try:
        maxfd = os.sysconf("SC_OPEN_MAX")
    except (AttributeError, ValueError):
        maxfd = 1024
       
    for fd in range(0, maxfd):
        try:
            os.close(fd)
        except OSError:
            pass

    os.open(os.devnull, os.O_RDWR)

    # Duplicate standard input to standard output and standard error.
    os.dup2(0, 1)
    os.dup2(0, 2)


def main(argv):
    """ The main daemon program.

    Keyword arguments:
    argv -- The arguments passed to the script.

    """
    # back up resolv.conf before we do anything else
    try:
        backup_location = wpath.varlib + 'resolv.conf.orig'
        # don't back up if .orig exists, probably there cause
        # wicd exploded
        if not os.path.exists(backup_location):
            shutil.copy2('/etc/resolv.conf', backup_location)
            os.chmod(backup_location, 0644)
    except IOError:
        print 'error backing up resolv.conf'

    do_daemonize = True
    redirect_stderr = True
    redirect_stdout = True
    auto_connect = True
    kill = False

    try:
        opts, args = getopt.getopt(sys.argv[1:], 'fenoahk',
                                   ['help', 'no-daemon', 'no-poll', 'no-stderr', 'no-stdout',
                                    'no-autoconnect', 'kill'])
    except getopt.GetoptError:
        # Print help information and exit
        usage()
        sys.exit(2)

    no_poll = False
    for o, a in opts:
        if o in ('-h', '--help'):
            usage()
            sys.exit()
        if o in ('-e', '--no-stderr'):
            redirect_stderr = False
        if o in ('-o', '--no-stdout'):
            redirect_stdout = False
        if o in ('-f', '--no-daemon'):
            do_daemonize = False
        if o in ('-a', '--no-autoconnect'):
            auto_connect = False
        if o in ('-n', '--no-poll'):
            no_poll = True
        if o in ('-k', '--kill'):
             kill = True

    if kill:
        try:
            f = open(wpath.pidfile)
        except:
            #print >> sys.stderr, "No wicd instance active, aborting."
            sys.exit(1)

        # restore resolv.conf on quit
        try:
            shutil.move(wpath.varlib + 'resolv.conf.orig', '/etc/resolv.conf')
            os.chmod('/etc/resolv.conf', 0644)
        except IOError:
            print 'error restoring resolv.conf'

        # connect to dbus, trigger a disconnect, then knock out the daemon
        from wicd import dbusmanager
        bus = dbusmanager.connect_to_dbus()
        dbus_ifaces = dbusmanager.get_dbus_ifaces()
        dbus_ifaces['daemon'].Disconnect()
        pid = int(f.readline())
        f.close()
        os.kill(pid,signal.SIGTERM)

        # quit, this should be the only option specified
        sys.exit(0)

    if os.path.exists(wpath.pidfile):
        print 'It seems like the daemon is already running.'
        print 'If it is not, please remove %s and try again.' % wpath.pidfile
        sys.exit(1)

    if not os.path.exists(wpath.networks):
        os.makedirs(wpath.networks)
   
    if do_daemonize: daemonize()

    if redirect_stderr or redirect_stdout:
        logpath = os.path.join(wpath.log, 'wicd.log')
        if not os.path.exists(wpath.log):
            os.makedirs(wpath.log)
            os.chmod(wpath.log, 0755)
        output = ManagedStdio(logpath)
        if os.path.exists(logpath):
            try:
                os.chmod(logpath, int(wpath.log_perms,8))
            except:
                print 'unable to chmod log file to %s' % wpath.log_perms

            try:
                if wpath.log_group:
                    import grp
                    group = grp.getgrnam(wpath.log_group)
                    os.chown(logpath, 0, group[2])
            except:
                print 'unable to chown log file to %s' % group[2]

    if redirect_stdout: sys.stdout = output
    if redirect_stderr: sys.stderr = output

    print '---------------------------'
    print 'wicd initializing...'
    print '---------------------------'

    print 'wicd is version', wpath.version, wpath.revision

    # Open the DBUS session
    bus = dbus.SystemBus()
    wicd_bus = dbus.service.BusName('org.wicd.daemon', bus=bus)
    daemon = WicdDaemon(wicd_bus, auto_connect=auto_connect)
    child_pid = None
    if not no_poll:
        child_pid = Popen([wpath.python, "-O", 
                          os.path.join(wpath.daemon, "monitor.py")],
                          shell=False, close_fds=True).pid
    atexit.register(on_exit, child_pid)

    # Enter the main loop
    mainloop = gobject.MainLoop()
    try:
        mainloop.run()
    except KeyboardInterrupt:
        pass
    daemon.DaemonClosing()

def on_exit(child_pid):
    """ Called when a SIGTERM is caught, kills monitor.py before exiting. """
    if child_pid:
        print 'Daemon going down, killing wicd-monitor...'
        try:
            os.kill(child_pid, signal.SIGTERM)
        except OSError:
            pass
    print 'Removing PID file...'
    if os.path.exists(wpath.pidfile):
        os.remove(wpath.pidfile)
    print 'Shutting down...'
    sys.exit(0)


if __name__ == '__main__':
    if os.getuid() != 0:
        print ("Root privileges are required for the daemon to run properly." +
               "  Exiting.")
        sys.exit(1)
    gobject.threads_init()
    main(sys.argv)



#!/usr/bin/env python

""" configmanager -- Wicd configuration file manager

Wrapper around ConfigParser for wicd, though it should be
reusable for other purposes as well.

"""

#
#   Copyright (C) 2008-2009 Adam Blackburn
#   Copyright (C) 2008-2009 Dan O'Reilly
#   Copyright (C) 2011      David Paleino
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import sys, os

from ConfigParser import RawConfigParser, ParsingError
import codecs

from wicd.misc import Noneify, to_unicode

from dbus import Int32

def sanitize_config_file(path):
    conf = open(path)
    newconf = ''
    for line in conf:
        if '[' in line or '=' in line:
            newconf += line
    conf.close()
    conf = open(path, 'w')
    conf.write(newconf)
    conf.close()

class ConfigManager(RawConfigParser):
    """ A class that can be used to manage a given configuration file. """
    def __init__(self, path, debug=False, mark_whitespace="`'`"):
        RawConfigParser.__init__(self)
        self.config_file = path
        self.debug = debug
        self.mrk_ws = mark_whitespace
        if os.path.exists(path):
            sanitize_config_file(path)
        try:
            self.read(path)
        except ParsingError, e:
            self.write()
            try:
                self.read(path)
            except ParsingError, p:
                print "Could not start wicd: %s" % p.message
                sys.exit(1)

    def __repr__(self):
        return self.config_file
    
    def __str__(self):
        return self.config_file
    
    def get_config(self):
        """ Returns the path to the loaded config file. """
        return self.config_file
        
    def set_option(self, section, option, value, write=False):
        """ Wrapper around ConfigParser.set

        Adds the option to write the config file change right away.
        Also forces all the values being written to type str, and
        adds the section the option should be written to if it
        doesn't exist already.
        
        """
        if not self.has_section(section):
            self.add_section(section)
        if isinstance(value, basestring):
            value = to_unicode(value)
            if value.startswith(' ') or value.endswith(' '):
                value = "%(ws)s%(value)s%(ws)s" % {"value" : value,
                                                   "ws" : self.mrk_ws}
        RawConfigParser.set(self, section, str(option), value)
        if write:
            self.write()

    def set(self, *args, **kargs):
        """ Calls the set_option method. """
        self.set_option(*args, **kargs)
        
    def get_option(self, section, option, default="__None__"):
        """ Wrapper around ConfigParser.get. 
        
        Automatically adds any missing sections, adds the ability
        to write a default value, and if one is provided prints if
        the default or a previously saved value is returned.
        
        """
        if not self.has_section(section):
            if default != "__None__":
                self.add_section(section)
            else:
                return None
    
        if self.has_option(section, option):
            ret = RawConfigParser.get(self, section, option)
            if (isinstance(ret, basestring) and ret.startswith(self.mrk_ws) 
                and ret.endswith(self.mrk_ws)):
                ret = ret[3:-3]
            ret = to_unicode(ret)
            if default:
                if self.debug:
                    # mask out sensitive information
                    if option in ['apsk', 'password', 'identity', 'private_key', \
                                  'private_key_passwd', 'key', 'passphrase']:
                        print ''.join(['found ', option, ' in configuration *****'])
                    else:
                        print ''.join(['found ', option, ' in configuration ', 
                                       str(ret)])
        else:
            if default != "__None__":
                print 'did not find %s in configuration, setting default %s' % (option, str(default))
                self.set(section, option, str(default), write=True)
                ret = default
            else:
                ret = None
        
        # Try to intelligently handle the type of the return value.
        try:
            if not ret.startswith('0') or len(ret) == 1:
                ret = int(ret)
        except (ValueError, TypeError, AttributeError):
            ret = Noneify(ret)
        # This is a workaround for a python-dbus issue on 64-bit systems.
        if isinstance(ret, (int, long)):
            try:
                Int32(ret)
            except OverflowError:
                ret = str(ret)
        return to_unicode(ret)
    
    def get(self, *args, **kargs):
        """ Calls the get_option method """
        return self.get_option(*args, **kargs)
    
    def _write_one(self):
        """ Writes the loaded config file to disk. """
        for section in self.sections():
            if not section:
                self.remove_section(section)
        configfile = open(self.config_file, 'w')
        RawConfigParser.write(self, configfile)
        configfile.close()
    
    def remove_section(self, section):
        """ Wrapper around the ConfigParser.remove_section() method.
        
        This method only calls the ConfigParser.remove_section() method
        if the section actually exists.
        
        """
        if self.has_section(section):
            RawConfigParser.remove_section(self, section)
            
    def reload(self):
        """ Re-reads the config file, in case it was edited out-of-band. """
        self.read(self.config_file)

    def read(self, path):
        """ Reads the config file specified by 'path' then reads all the
        files in the directory obtained by adding '.d' to 'path'. The files
        in the '.d' directory are read in normal sorted order and section
        entries in these files override entries in the main file.
        """
        if os.path.exists(path):
            RawConfigParser.readfp(self, codecs.open(path, 'r', 'utf-8'))

        path_d = path + ".d"
        files = []

        if os.path.exists(path_d):
            files = [ os.path.join(path_d, f) for f in os.listdir(path_d) ]
            files.sort()

        for fname in files:
            p = RawConfigParser()
            p.readfp(codecs.open(fname, 'r', 'utf-8'))
            for section_name in p.sections():
                # New files override old, so remove first to avoid DuplicateSectionError.
                self.remove_section(section_name)
                self.add_section(section_name)
                for (name, value) in p.items(section_name):
                    self.set(section_name, name, value)
                # Store the filename this section was read from.
                self.set(section_name, '_filename_', fname)


    def _copy_section(self, name):
        p = ConfigManager("", self.debug, self.mrk_ws)
        p.add_section(name)
        for (iname, value) in self.items(name):
            p.set(name, iname, value)
        # Store the filename this section was read from.
        p.config_file = p.get_option(name, '_filename_', p.config_file)
        p.remove_option(name, '_filename_')
        return p

    def write(self):
        """ Writes the loaded config file to disk. """
        in_this_file = []
        for sname in sorted(self.sections()):
            fname = self.get_option(sname, '_filename_')
            if fname and fname != self.config_file:
                # Write sections from other files
                section = self._copy_section(sname)
                section._write_one()
            else:
                # Save names of local sections
                in_this_file.append(sname)

        # Make an instance with only these sections
        p = ConfigManager("", self.debug, self.mrk_ws)
        p.config_file = self.config_file
        for sname in in_this_file:
            p.add_section(sname)
            for (iname, value) in self.items(sname):
                p.set(sname, iname, value)
            p.remove_option(sname, '_filename_')
        p._write_one()




#!/usr/bin/env python
"""
    netentry_curses -- everyone's favorite networks settings dialogs... in text
    form!
"""

#       Copyright (C) 2009 Andrew Psaltis

#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

import urwid
from curses_misc import TextDialog,DynWrap,MaskingEdit,ComboBox,error
import wicd.misc as misc
from wicd.misc import noneToString, stringToNone, noneToBlankString, to_bool

from wicd.translations import language, _
import os

daemon = None
wired = None
wireless = None
# Call this first!
def dbus_init(dbus_ifaces):
    global daemon,wired,wireless
    daemon = dbus_ifaces['daemon']
    wired = dbus_ifaces['wired']
    wireless = dbus_ifaces['wireless']

# Both the wired and the wireless settings preferences dialogs use some of the
# same fields.
# This will be used to produce the individual network settings dialogs way far below
class AdvancedSettingsDialog(urwid.WidgetWrap):
    def __init__(self):
        self.ui=None

        static_ip_t = _('Use Static IPs')
        ip_t        = ('editcp',_('IP')+':     ')
        netmask_t   = ('editcp',_('Netmask')+':')
        gateway_t   = ('editcp',_('Gateway')+':')

        use_static_dns_t = _('Use Static DNS')
        use_global_dns_t = _('Use global DNS servers')
        dns_dom_t    = ('editcp',_('DNS domain')+':   ')
        search_dom_t = ('editcp',_('Search domain')+':')
        dns1_t       = ('editcp',_('DNS server')+ ' 1'+':'+' '*8)
        dns2_t       = ('editcp',_('DNS server')+ ' 2'+':'+' '*8)
        dns3_t       = ('editcp',_('DNS server')+ ' 3'+':'+' '*8)
        
        use_dhcp_h_t = _('Use DHCP Hostname')
        dhcp_h_t     = ('editcp',_('DHCP Hostname')+': ')
        
        cancel_t = _('Cancel')
        ok_t = _('OK')
        
        self.static_ip_cb = urwid.CheckBox(static_ip_t,
                on_state_change=self.static_ip_toggle)
        self.ip_edit      = DynWrap(urwid.Edit(ip_t),False)
        self.netmask_edit = DynWrap(urwid.Edit(netmask_t),False)
        self.gateway_edit = DynWrap(urwid.Edit(gateway_t),False)


        self.static_dns_cb = DynWrap(urwid.CheckBox(use_static_dns_t,
                on_state_change=self.dns_toggle),True,('body','editnfc'),None)
        self.global_dns_cb = DynWrap(urwid.CheckBox(use_global_dns_t,
                on_state_change=self.dns_toggle),False,('body','editnfc'),None)
        self.checkb_cols = urwid.Columns([self.static_dns_cb,
                                     self.global_dns_cb])
        self.dns_dom_edit      = DynWrap(urwid.Edit(dns_dom_t)   ,False)
        self.search_dom_edit   = DynWrap(urwid.Edit(search_dom_t),False)
        self.dns1              = DynWrap(urwid.Edit(dns1_t)      ,False)
        self.dns2              = DynWrap(urwid.Edit(dns2_t)      ,False)
        self.dns3              = DynWrap(urwid.Edit(dns3_t)      ,False)

        self.use_dhcp_h        = urwid.CheckBox(use_dhcp_h_t,False,on_state_change=self.use_dhcp_h_toggle)
        self.dhcp_h            = DynWrap(urwid.Edit(dhcp_h_t),False)

        _blank = urwid.Text('')

        walker = urwid.SimpleListWalker([self.static_ip_cb,
                                         self.ip_edit,
                                         self.netmask_edit,
                                         self.gateway_edit,
                                         _blank,
                                         self.checkb_cols,
                                         self.dns_dom_edit,self.search_dom_edit,
                                         self.dns1,self.dns2,self.dns3,
                                         _blank,
                                         self.use_dhcp_h,
                                         self.dhcp_h,
                                         _blank
                                        ])



        self._listbox = urwid.ListBox(walker)
        self._frame = urwid.Frame(self._listbox)
        self.__super.__init__(self._frame)
    
    def use_dhcp_h_toggle(self,checkb,new_state,user_data=None):
        self.dhcp_h.set_sensitive(new_state)

    def static_ip_toggle(self,checkb,new_state,user_data=None):
        for w in [ self.ip_edit,self.netmask_edit,self.gateway_edit ]:
            w.set_sensitive(new_state)
        self.static_dns_cb.set_state(new_state)
        self.static_dns_cb.set_sensitive(not new_state)
        if new_state:
            self.checkb_cols.set_focus(self.global_dns_cb)
        else:
            self.checkb_cols.set_focus(self.static_dns_cb)

        
    def dns_toggle(self,checkb,new_state,user_data=None):
        if checkb == self.static_dns_cb.get_w():
            for w in [ self.dns_dom_edit,self.search_dom_edit,
                    self.dns1,self.dns2,self.dns3 ]:
                w.set_sensitive(new_state)
            if not new_state:
                self.global_dns_cb.set_state(False,do_callback=False)
            self.global_dns_cb.set_sensitive(new_state)
        # use_global_dns_cb is DynWrapped
        if checkb == self.global_dns_cb.get_w():
            for w in [self.dns_dom_edit,self.search_dom_edit,
                    self.dns1,self.dns2,self.dns3 ]:
                w.set_sensitive(not new_state)

    # Code totally yanked from netentry.py
    def save_settings(self):
        """ Save settings common to wired and wireless settings dialogs. """
        if self.static_ip_cb.get_state():
            for i in [self.ip_edit,self.netmask_edit,self.gateway_edit]:
                i.set_edit_text(i.get_edit_text().strip())

            self.set_net_prop("ip", noneToString(self.ip_edit.get_edit_text()))
            self.set_net_prop("netmask", noneToString(self.netmask_edit.get_edit_text()))
            self.set_net_prop("gateway", noneToString(self.gateway_edit.get_edit_text()))
        else:
            self.set_net_prop("ip", '')
            self.set_net_prop("netmask", '')
            self.set_net_prop("gateway", '')

        if self.static_dns_cb.get_state() and \
           not self.global_dns_cb.get_state():
            self.set_net_prop('use_static_dns', True)
            self.set_net_prop('use_global_dns', False)
            # Strip addressses before checking them in the daemon.
            for i in [self.dns1, self.dns2,
                      self.dns3,self.dns_dom_edit, self.search_dom_edit]:
                i.set_edit_text(i.get_edit_text().strip())
            self.set_net_prop('dns_domain', noneToString(self.dns_dom_edit.get_edit_text()))
            self.set_net_prop("search_domain", noneToString(self.search_dom_edit.get_edit_text()))
            self.set_net_prop("dns1", noneToString(self.dns1.get_edit_text()))
            self.set_net_prop("dns2", noneToString(self.dns2.get_edit_text()))
            self.set_net_prop("dns3", noneToString(self.dns3.get_edit_text()))
        elif self.static_dns_cb.get_state() and \
             self.global_dns_cb.get_state():
            self.set_net_prop('use_static_dns', True)
            self.set_net_prop('use_global_dns', True)
        else:
            self.set_net_prop('use_static_dns', False)
            self.set_net_prop('use_global_dns', False)
            self.set_net_prop('dns_domain', '')
            self.set_net_prop("search_domain", '')
            self.set_net_prop("dns1", '')
            self.set_net_prop("dns2", '')
            self.set_net_prop("dns3", '')
        self.set_net_prop('dhcphostname',self.dhcp_h.get_edit_text())
        self.set_net_prop('usedhcphostname',self.use_dhcp_h.get_state())

    # Prevent comboboxes from dying.
    def ready_widgets(self,ui,body):
        self.ui = ui
        self.body = body
        self.encryption_combo.build_combobox(body,ui,14)
        self.change_encrypt_method()

    def combo_on_change(self,combobox,new_index,user_data=None):
        self.change_encrypt_method()
    
    # More or less ripped from netentry.py
    def change_encrypt_method(self):
        #self.lbox_encrypt = urwid.ListBox()
        self.encryption_info = {}
        wid,ID = self.encryption_combo.get_focus()
        methods = self.encrypt_types

        if self._w.body.body.__contains__(self.pile_encrypt):
            self._w.body.body.pop(self._w.body.body.__len__()-1)

        # If nothing is selected, select the first entry.
        if ID == -1:
            self.encryption_combo.set_focus(0)
            ID = 0

        theList = []
        for type_ in ['required', 'optional']:
            fields = methods[ID][type_]
            for field in fields:
                try:
                    edit = MaskingEdit(('editcp',language[field[1].lower().replace(' ','_')]+': '))
                except KeyError:
                    edit = MaskingEdit(('editcp',field[1].replace(' ','_')+': '))
                edit.set_mask_mode('no_focus')
                theList.append(edit)
                # Add the data to any array, so that the information
                # can be easily accessed by giving the name of the wanted
                # data.
                self.encryption_info[field[0]] = [edit, type_]
                
                if self.wired:
                    edit.set_edit_text(noneToBlankString(
                        wired.GetWiredProperty(field[0])))
                else:
                    edit.set_edit_text(noneToBlankString(
                        wireless.GetWirelessProperty(self.networkid, field[0])))

        #FIXME: This causes the entire pile to light up upon use.
        # Make this into a listbox?
        self.pile_encrypt = DynWrap(urwid.Pile(theList),attrs=('editbx','editnfc'))
        
        self.pile_encrypt.set_sensitive(self.encryption_chkbox.get_state())
        
        self._w.body.body.insert(self._w.body.body.__len__(),self.pile_encrypt)
        #self._w.body.body.append(self.pile_encrypt)
    
    def encryption_toggle(self,chkbox,new_state,user_data=None):
        self.encryption_combo.set_sensitive(new_state)
        self.pile_encrypt.set_sensitive(new_state)

class WiredSettingsDialog(AdvancedSettingsDialog):
    def __init__(self,name,parent):
        global wired, daemon
        AdvancedSettingsDialog.__init__(self)
        self.wired = True
        
        self.set_default = urwid.CheckBox(_('Use as default profile (overwrites any previous default)'))
        #self.cur_default = 
        # Add widgets to listbox
        self._w.body.body.append(self.set_default)
        
        self.parent = parent
        encryption_t = _('Use Encryption')
        
        self.encryption_chkbox = urwid.CheckBox(encryption_t,on_state_change=self.encryption_toggle)
        self.encryption_combo = ComboBox(callback=self.combo_on_change)
        self.pile_encrypt = None
        # _w is a Frame, _w.body is a ListBox, _w.body.body is the ListWalker :-)
        self._listbox.body.append(self.encryption_chkbox)
        self._listbox.body.append(self.encryption_combo)
        self.encrypt_types = misc.LoadEncryptionMethods(wired = True)
        self.set_values()
        
        self.prof_name = name
        title = _('Configuring preferences for wired profile "$A"').replace('$A',self.prof_name)
        self._w.header = urwid.Text( ('header',title),align='right' )

        self.set_values()
    def set_net_prop(self,option,value): 
        wired.SetWiredProperty(option,value)
    def set_values(self):
        self.ip_edit.set_edit_text(self.format_entry("ip"))
        self.netmask_edit.set_edit_text(self.format_entry("netmask"))
        self.gateway_edit.set_edit_text(self.format_entry("gateway"))

        self.global_dns_cb.set_state(bool(wired.GetWiredProperty('use_global_dns')))
        self.static_dns_cb.set_state(bool(wired.GetWiredProperty('use_static_dns')))
        
        # Set static ip checkbox.  Forgot to do this the first time.
        if stringToNone(self.ip_edit.get_edit_text()):
            self.static_ip_cb.set_state(True)
        self.dns1.set_edit_text(self.format_entry( "dns1"))
        self.dns2.set_edit_text(self.format_entry( "dns2"))
        self.dns3.set_edit_text(self.format_entry( "dns3"))
        self.dns_dom_edit.set_edit_text(self.format_entry("dns_domain"))
        self.search_dom_edit.set_edit_text(self.format_entry("search_domain"))

        self.set_default.set_state(to_bool(wired.GetWiredProperty("default")))

        # Throw the encryption stuff into a list
        list = []
        activeID = -1  # Set the menu to this item when we are done
        for x, enc_type in enumerate(self.encrypt_types):
            list.append(enc_type['name'])
            if enc_type['type'] == wired.GetWiredProperty("enctype"):
                activeID = x
        self.encryption_combo.set_list(list)

        self.encryption_combo.set_focus(activeID)
        if wired.GetWiredProperty("encryption_enabled"):
            self.encryption_chkbox.set_state(True,do_callback=False)
            self.encryption_combo.set_sensitive(True)
            #self.lbox_encrypt_info.set_sensitive(True)
        else:
            self.encryption_combo.set_focus(0)
            self.encryption_combo.set_sensitive(False)

        self.change_encrypt_method()

        dhcphname = wired.GetWiredProperty("dhcphostname")
        if dhcphname is None:
            dhcphname = os.uname()[1]

        self.use_dhcp_h.set_state(bool(wired.GetWiredProperty('usedhcphostname')))
        self.dhcp_h.set_sensitive(self.use_dhcp_h.get_state())
        self.dhcp_h.set_edit_text(unicode(dhcphname))

    def save_settings(self):
        # Check encryption info
        if self.encryption_chkbox.get_state():
            encrypt_info = self.encryption_info
            encrypt_methods = self.encrypt_types
            self.set_net_prop("enctype",
                               encrypt_methods[self.encryption_combo.get_focus()[1] ]['type'])
            self.set_net_prop("encryption_enabled", True)
            # Make sure all required fields are filled in.
            for entry_info in encrypt_info.itervalues():
                if entry_info[0].get_edit_text() == "" \
                    and entry_info[1] == 'required':
                    error(self.ui, self.parent,"%s (%s)" \
                            % (_('Required encryption information is missing.'),
                                entry_info[0].get_caption()[0:-2] )
                          )
                    return False

            for entry_key, entry_info in encrypt_info.iteritems():
                self.set_net_prop(entry_key, noneToString(entry_info[0].
                                                   get_edit_text()))
        else:
            self.set_net_prop("enctype", "None")
            self.set_net_prop("encryption_enabled", False)
        
        AdvancedSettingsDialog.save_settings(self)
        if self.set_default.get_state():
            wired.UnsetWiredDefault()
        if self.set_default.get_state():
            bool = True
        else:
            bool = False
        wired.SetWiredProperty("default",bool)
        wired.SaveWiredNetworkProfile(self.prof_name)
        return True

    def format_entry(self, label):
        """ Helper method to fetch and format wired properties. """
        return noneToBlankString(wired.GetWiredProperty(label))
    def prerun(self,ui,dim,display):
        pass

########################################

class WirelessSettingsDialog(AdvancedSettingsDialog):
    def __init__(self,networkID,parent):
        global wireless, daemon
        AdvancedSettingsDialog.__init__(self)
        self.wired = False
        
        self.networkid = networkID
        self.parent = parent
        global_settings_t = _('Use these settings for all networks sharing this essid')
        encryption_t = _('Use Encryption')
        autoconnect_t = _('Automatically connect to this network')
        
        self.global_settings_chkbox = urwid.CheckBox(global_settings_t)
        self.encryption_chkbox = urwid.CheckBox(encryption_t,on_state_change=self.encryption_toggle)
        self.encryption_combo = ComboBox(callback=self.combo_on_change)
        self.autoconnect_chkbox = urwid.CheckBox(autoconnect_t)
        self.pile_encrypt = None
        # _w is a Frame, _w.body is a ListBox, _w.body.body is the ListWalker :-)
        self._listbox.body.append(self.global_settings_chkbox)
        self._listbox.body.append(self.autoconnect_chkbox)
        self._listbox.body.append(self.encryption_chkbox)
        self._listbox.body.append(self.encryption_combo)
        self.encrypt_types = misc.LoadEncryptionMethods()
        self.set_values()

        title = _('Configuring preferences for wireless network "$A" ($B)').replace('$A',wireless.GetWirelessProperty(networkID,'essid')).replace('$B',wireless.GetWirelessProperty(networkID,'bssid'))
        self._w.header = urwid.Text(('header',title),align='right' )

    def set_values(self):
        """ Set the various network settings to the right values. """
        networkID = self.networkid
        self.ip_edit.set_edit_text(self.format_entry(networkID,"ip"))
        self.netmask_edit.set_edit_text(self.format_entry(networkID,"netmask"))
        self.gateway_edit.set_edit_text(self.format_entry(networkID,"gateway"))

        self.global_dns_cb.set_state(bool(wireless.GetWirelessProperty(networkID,
                                                                  'use_global_dns')))
        self.static_dns_cb.set_state(bool(wireless.GetWirelessProperty(networkID,
                                                                  'use_static_dns')))
        
        if stringToNone(self.ip_edit.get_edit_text()):
            self.static_ip_cb.set_state(True)
        self.dns1.set_edit_text(self.format_entry(networkID, "dns1"))
        self.dns2.set_edit_text(self.format_entry(networkID, "dns2"))
        self.dns3.set_edit_text(self.format_entry(networkID, "dns3"))
        self.dns_dom_edit.set_edit_text(self.format_entry(networkID, "dns_domain"))
        self.search_dom_edit.set_edit_text(self.format_entry(networkID, "search_domain"))
        
        self.autoconnect_chkbox.set_state(to_bool(self.format_entry(networkID, "automatic")))

        #self.reset_static_checkboxes()
        self.encryption_chkbox.set_state(bool(wireless.GetWirelessProperty(networkID,
            'encryption')),do_callback=False)
        self.global_settings_chkbox.set_state(bool(wireless.GetWirelessProperty(networkID
            ,'use_settings_globally')))

        # Throw the encryption stuff into a list
        list = []
        activeID = -1  # Set the menu to this item when we are done
        for x, enc_type in enumerate(self.encrypt_types):
            list.append(enc_type['name'])
            if enc_type['type'] == wireless.GetWirelessProperty(networkID, "enctype"):
                activeID = x
        self.encryption_combo.set_list(list)

        self.encryption_combo.set_focus(activeID)
        if activeID != -1:
            self.encryption_chkbox.set_state(True,do_callback=False)
            self.encryption_combo.set_sensitive(True)
            #self.lbox_encrypt_info.set_sensitive(True)
        else:
            self.encryption_combo.set_focus(0)

        self.change_encrypt_method()
        dhcphname = wireless.GetWirelessProperty(networkID,"dhcphostname")
        if dhcphname is None:
            dhcphname = os.uname()[1]
        self.use_dhcp_h.set_state(bool(wireless.GetWirelessProperty(networkID,'usedhcphostname')))
        self.dhcp_h.set_sensitive(self.use_dhcp_h.get_state())
        self.dhcp_h.set_edit_text(unicode(dhcphname))
        

    def set_net_prop(self, option, value):
        """ Sets the given option to the given value for this network. """
        wireless.SetWirelessProperty(self.networkid, option, value)

    def format_entry(self, networkid, label):
        """ Helper method for fetching/formatting wireless properties. """
        return noneToBlankString(wireless.GetWirelessProperty(networkid, label))

    # Ripped from netentry.py
    def save_settings(self):
        # Check encryption info
        if self.encryption_chkbox.get_state():
            encrypt_info = self.encryption_info
            encrypt_methods = self.encrypt_types
            self.set_net_prop("enctype",
                               encrypt_methods[self.encryption_combo.get_focus()[1] ]['type'])
            # Make sure all required fields are filled in.
            for entry_info in encrypt_info.itervalues():
                if entry_info[0].get_edit_text() == "" \
                    and entry_info[1] == 'required':
                    error(self.ui, self.parent,"%s (%s)" \
                            % (_('Required encryption information is missing.'),
                                entry_info[0].get_caption()[0:-2] )
                          )
                    return False

            for entry_key, entry_info in encrypt_info.iteritems():
                self.set_net_prop(entry_key, noneToString(entry_info[0].
                                                   get_edit_text()))
        elif not self.encryption_chkbox.get_state() and \
             wireless.GetWirelessProperty(self.networkid, "encryption"):
            # Encrypt checkbox is off, but the network needs it.
            error(self.ui, self.parent, _('This network requires encryption to be enabled.'))
            return False
        else:
            self.set_net_prop("enctype", "None")
        AdvancedSettingsDialog.save_settings(self)

        # Save the autoconnect setting.  This is not where it originally was
        # in the GTK UI.
        self.set_net_prop("automatic",self.autoconnect_chkbox.get_state())
        
        if self.global_settings_chkbox.get_state():
            self.set_net_prop('use_settings_globally', True)
        else:
            self.set_net_prop('use_settings_globally', False)
            wireless.RemoveGlobalEssidEntry(self.networkid)
            
        wireless.SaveWirelessNetworkProfile(self.networkid)
        return True



#!/usr/bin/env python
# -* coding: utf-8 -*-

""" translations -- module for handling the translation strings for wicd. """
#
#   Copyright (C) 2007 - 2009 Adam Blackburn
#   Copyright (C) 2007 - 2009 Dan O'Reilly
#   Copyright (C) 2009        Andrew Psaltis
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import locale
import os
import wpath
import gettext


def get_gettext():
    """ Set up gettext for translations. """
    # Borrowed from an excellent post on how to do this at
    # http://www.learningpython.com/2006/12/03/translating-your-pythonpygtk-application/
    local_path = wpath.translations
    langs = []
    osLanguage = os.environ.get('LANGUAGE', None)
    if osLanguage:
        langs += osLanguage.split(":")
    osLanguage = None
    osLanguage = os.environ.get('LC_MESSAGES', None)
    if osLanguage:
        langs += osLanguage.split(":")
    try:
        # This avoids a bug: locale.getdefaultlocale() prefers
        # LC_CTYPE over LANG/LANGUAGE
        lc, encoding = locale.getdefaultlocale(envvars=('LC_MESSAGES', 
                                                        'LC_ALL', 'LANG', 
                                                        'LANGUAGE'))
    except ValueError, e:
        print str(e)
        print "Default locale unavailable, falling back to en_US"
    if (lc):
        langs += [lc]
    langs += ["en_US"]
    lang = gettext.translation('wicd', local_path, languages=langs, 
                               fallback=True)
    _ = lang.gettext
    return _

_ = get_gettext()


# language[] should contain only strings in encryption templates, which
# can't otherwise be translated, at least with the current templating
# scheme.

language = {}

# FIXME: these were present in wicd 1.7.0, can't find where they are.
# Leaving here for future reference, they should be removed whenever
# possible.
#language['cannot_start_daemon'] = _('''Unable to connect to wicd daemon DBus interface. This typically means there was a problem starting the daemon. Check the wicd log for more information.''')
#language['backend_alert'] = _('''Changes to your backend won't occur until the daemon is restarted.''')
#language['about_help'] = _('''Stop a network connection in progress''')
#language['connect'] = _('''Connect''')

# from templates, dict populated with:
# grep -R "*" encryption/templates/ | tr " " "\n" | grep "^*" | sed -e "s/*//"| sort -u | tr [A-Z] [a-z]
language['authentication'] = _('Authentication')
language['domain'] = _('Domain')
language['identity'] = _('Identity')
language['key'] = _('Key')
language['passphrase'] = _('Passphrase')
language['password'] = _('Password')
language['path_to_ca_cert'] = _('Path to CA cert')
language['path_to_client_cert'] = _('Path to client cert')
language['path_to_pac_file'] = _('Path to PAC file')
language['preshared_key'] = _('Preshared key')
language['private_key'] = _('Private key')
language['private_key_password'] = _('Private key password')
language['username'] = _('Username')



#!/usr/bin/env python

""" Suspends the wicd daemon.

Sets a flag in the daemon that will stop it from monitoring network status.
Used for when a laptop enters hibernation/suspension.

"""

#
#   Copyright (C) 2007 - 2009 Adam Blackburn
#   Copyright (C) 2007 - 2009 Dan O'Reilly
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import dbus
import dbus.service
import sys

try:
    bus = dbus.SystemBus()
    proxy_obj = bus.get_object('org.wicd.daemon', '/org/wicd/daemon')
    daemon = dbus.Interface(proxy_obj, 'org.wicd.daemon')
except Exception, e:
    print>>sys.stderr, "Exception caught: %s" % str(e)
    print>>sys.stderr, 'Could not connect to daemon.'
    sys.exit(1)
    


if __name__ == '__main__':
    try:
        daemon.Disconnect()
        daemon.SetForcedDisconnect(False)
        daemon.SetSuspend(True)
    except Exception, e:
        print>>sys.stderr, "Exception caught: %s" % str(e)
        print>>sys.stderr, 'Error setting suspend.'
        sys.exit(2)
        



""" misc - miscellaneous functions for wicd

This module contains a large variety of utility functions used
throughout wicd.

"""

#
#   Copyright (C) 2007 - 2009 Adam Blackburn
#   Copyright (C) 2007 - 2009 Dan O'Reilly
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License Version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os
import locale
import sys
import re
import gobject
from threading import Thread
from subprocess import Popen, STDOUT, PIPE, call
from commands import getoutput
from itertools import repeat, chain, izip
from pipes import quote

from wicd.translations import _

# wicd imports
import wpath

# Connection state constants
NOT_CONNECTED = 0
CONNECTING = 1
WIRELESS = 2
WIRED = 3
SUSPENDED = 4

# Automatic app selection constant
AUTO = 0

# DHCP Clients
DHCLIENT = 1
DHCPCD = 2
PUMP = 3
UDHCPC = 4

# Link detection tools
ETHTOOL = 1
MIITOOL = 2

# Route flushing tools
IP = 1
ROUTE = 2

# Graphical sudo apps
GKSUDO = 1
KDESU = 2
KTSUSS = 3
_sudo_dict = { 
    AUTO : "",
    GKSUDO : "gksudo",
    KDESU : "kdesu",
    KTSUSS: "ktsuss",
}

_status_dict = {
    'aborted': _('Connection Cancelled'),
    'association_failed': _('Connection failed: Could not contact the wireless access point.'),
    'bad_pass': _('Connection Failed: Bad password'),
    'configuring_interface': _('Configuring wireless interface...'),
    'dhcp_failed': _('Connection Failed: Unable to Get IP Address'),
    'done': _('Done connecting...'),
    'failed': _('Connection Failed.'),
    'flushing_routing_table': _('Flushing the routing table...'),
    'generating_psk': _('Generating PSK...'),
    'generating_wpa_config': _('Generating WPA configuration file...'),
    'interface_down': _('Putting interface down...'),
    'interface_up': _('Putting interface up...'),
    'no_dhcp_offers': _('Connection Failed: No DHCP offers received.'),
    'resetting_ip_address': _('Resetting IP address...'),
    'running_dhcp': _('Obtaining IP address...'),
    'setting_broadcast_address': _('Setting broadcast address...'),
    'setting_static_dns': _('Setting static DNS servers...'),
    'setting_static_ip': _('Setting static IP addresses...'),
    'success': _('Connection successful.'),
    'validating_authentication': _('Validating authentication...'),
    'verifying_association': _('Verifying access point association...'),
}

class WicdError(Exception):
    pass
    

def Run(cmd, include_stderr=False, return_pipe=False,
        return_obj=False, return_retcode=True):
    """ Run a command.

    Runs the given command, returning either the output
    of the program, or a pipe to read output from.

    keyword arguments --
    cmd - The command to execute
    include_std_err - Boolean specifying if stderr should
                      be included in the pipe to the cmd.
    return_pipe - Boolean specifying if a pipe to the
                  command should be returned.  If it is
                  False, all that will be returned is
                  one output string from the command.
    return_obj - If True, Run will return the Popen object
                 for the command that was run.

    """
    if not isinstance(cmd, list):
        cmd = to_unicode(str(cmd))
        cmd = cmd.split()
    if include_stderr:
        err = STDOUT
        fds = True
    else:
        err = None
        fds = False
    if return_obj:
        std_in = PIPE
    else:
        std_in = None
    
    # We need to make sure that the results of the command we run
    # are in English, so we set up a temporary environment.
    tmpenv = os.environ.copy()
    tmpenv["LC_ALL"] = "C"
    tmpenv["LANG"] = "C"
    
    try:
        f = Popen(cmd, shell=False, stdout=PIPE, stdin=std_in, stderr=err,
                  close_fds=fds, cwd='/', env=tmpenv)
    except OSError, e:
        print "Running command %s failed: %s" % (str(cmd), str(e))
        return ""
        
    if return_obj:
        return f
    if return_pipe:
        return f.stdout
    else:
        return f.communicate()[0]
    
def LaunchAndWait(cmd):
    """ Launches the given program with the given arguments, then blocks.

    cmd : A list contained the program name and its arguments.

    returns: The exit code of the process.
    
    """
    if not isinstance(cmd, list):
        cmd = to_unicode(str(cmd))
        cmd = cmd.split()
    p = Popen(cmd, shell=False, stdout=PIPE, stderr=STDOUT, stdin=None)
    return p.wait()

def IsValidIP(ip):
    """ Make sure an entered IP is valid. """
    if not ip: return False

    ipNumbers = ip.split('.')
    if len(ipNumbers) < 4:
        return False

    for number in ipNumbers:
        if not number.isdigit() or int(number) > 255:
            return False

    return ipNumbers

def PromptToStartDaemon():
    """ Prompt the user to start the daemon """
    daemonloc = wpath.sbin + 'wicd'
    sudo_prog = choose_sudo_prog()
    if not sudo_prog:
        return False
    if "gksu" in sudo_prog or "ktsuss" in sudo_prog:
        msg = '--message'
    else:
        msg = '--caption'
    sudo_args = [sudo_prog, msg, 
                 _("Wicd needs to access your computer's network cards."),
                 daemonloc]
    os.spawnvpe(os.P_WAIT, sudo_prog, sudo_args, os.environ)
    return True

def RunRegex(regex, string):
    """ runs a regex search on a string """
    m = regex.search(string)
    if m:
        return m.groups()[0]
    else:
        return None

def WriteLine(my_file, text):
    """ write a line to a file """
    my_file.write(text + "\n")

def ExecuteScripts(scripts_dir, verbose=False, extra_parameters=()):
    """ Execute every executable file in a given directory. """
    if not os.path.exists(scripts_dir):
        return
    for obj in sorted(os.listdir(scripts_dir)):
        if obj.startswith(".") or obj.endswith(("~", ".new", ".orig")):
            continue
        obj = os.path.abspath(os.path.join(scripts_dir, obj))
        if os.path.isfile(obj) and os.access(obj, os.X_OK):
            ExecuteScript(os.path.abspath(obj), verbose=verbose,
                          extra_parameters=extra_parameters)

def ExecuteScript(script, verbose=False, extra_parameters=()):
    """ Execute a command and send its output to the bit bucket. """
    extra_parameters = [ quote(s) for s in extra_parameters ]
    params = ' '.join(extra_parameters)
    # escape script name
    script = quote(script)
    if verbose:
        print "Executing %s with params %s" % (script, params)
    ret = call('%s %s > /dev/null 2>&1' % (script, params), shell=True)
    if verbose:
        print "%s returned %s" % (script, ret)

def ReadFile(filename):
    """ read in a file and return it's contents as a string """
    if not os.path.exists(filename):
        return None
    my_file = open(filename,'r')
    data = my_file.read().strip()
    my_file.close()
    return str(data)

def to_bool(var):
    """ Convert a string to type bool, but make "False"/"0" become False. """
    if var in ("False", "0"):
        var = False
    else:
        var = bool(var)
    return var

def Noneify(variable):
    """ Convert string types to either None or booleans"""
    #set string Nones to real Nones
    if variable in ("None", "", None):
        return None
    if variable in ("False", "0"):
        return False
    if variable in ("True", "1"):
        return True
    return variable

def ParseEncryption(network):
    """ Parse through an encryption template file

    Parses an encryption template, reading in a network's info
    and creating a config file for it

    """
    enctemplate = open(wpath.encryption + network["enctype"])
    template = enctemplate.readlines()
    if network.get('essid'):
        config_file = "ap_scan=1\n"
    else:
        config_file = "ap_scan=0\n"
    should_replace = False
    for index, line in enumerate(template):
        if not should_replace:
            if line.strip().startswith('---'):
                should_replace = True
        else:
            if line.strip().startswith("}"):
                # This is the last line, so we just write it.
                config_file = ''.join([config_file, line])
            elif "$_" in line: 
                cur_val = re.findall('\$_([A-Z0-9_]+)', line)
                if cur_val:
                    if cur_val[0] == 'SCAN':
                        #TODO should this be hardcoded?
                        line = line.replace("$_SCAN", "1")
                        config_file = ''.join([config_file, line])
                    else:
                        rep_val = network.get(cur_val[0].lower())
                        if rep_val:
                            line = line.replace("$_%s" % cur_val[0], 
                                                str(rep_val))
                            config_file = ''.join([config_file, line])
                        else:
                            print "Ignoring template line: '%s'" % line
                else:
                    print "Weird parsing error occurred"
            else:  # Just a regular entry.
                config_file = ''.join([config_file, line])

    # Write the data to the files then chmod them so they can't be read 
    # by normal users.
    if network.get('bssid'):
        file_name = network['bssid'].replace(":", "").lower()
    else:
        file_name = 'wired'
    file_loc = os.path.join(wpath.networks, file_name)
    f = open(file_loc, "w")
    os.chmod(file_loc, 0600)
    os.chown(file_loc, 0, 0)
    # We could do this above, but we'd like to read protect
    # them before we write, so that it can't be read.
    f.write(config_file)
    f.close()

def LoadEncryptionMethods(wired = False):
    """ Load encryption methods from configuration files

    Loads all the encryption methods from the template files
    in /encryption/templates into a data structure.  To be
    loaded, the template must be listed in the "active" file.

    """
    if wired:
        active_fname = "active_wired"
    else:
        active_fname = "active"
    try:
        enctypes = open(wpath.encryption + active_fname,"r").readlines()
    except IOError, e:
        print "Fatal Error: template index file is missing."
        raise IOError(e)
    
    # Parse each encryption method
    encryptionTypes = []
    for enctype in enctypes:
        parsed_template = _parse_enc_template(enctype.strip())
        if parsed_template:
            encryptionTypes.append(parsed_template)
    return encryptionTypes

def __parse_field_ent(fields, field_type='require'):
    fields = fields.split(" ")
    ret = []
    # We need an even number of entries in the line for it to be valid.
    if (len(fields) % 2) != 0:
        return None
    else:
        for val, disp_val in grouper(2, fields, fillvalue=None):
            if val.startswith("*") or not disp_val.startswith("*"):
                return None
            ret.append([val, disp_val[1:]])
        return ret

def _parse_enc_template(enctype):
    """ Parse an encryption template. """
    def parse_ent(line, key):
        return line.replace(key, "").replace("=", "").strip()

    try:
        f = open(os.path.join(wpath.encryption, enctype), "r")
    except IOError:
        print "Failed to open template file %s" % enctype
        return None

    cur_type = {}
    cur_type["type"] = enctype
    cur_type["fields"] = []
    cur_type['optional'] = []
    cur_type['required'] = []
    cur_type['protected'] = []
    cur_type['name'] = ""
    for index, line in enumerate(f):
        if line.startswith("name") and not cur_type["name"]:
            cur_type["name"] = parse_ent(line, "name")
        elif line.startswith("require"):
            cur_type["required"] = __parse_field_ent(parse_ent(line, "require"))
            if not cur_type["required"]:
                # An error occured parsing the require line.
                print "Invalid 'required' line found in template %s" % enctype
                continue
        elif line.startswith("optional"):
            cur_type["optional"] = __parse_field_ent(parse_ent(line,
                                                               "optional"),
                                                     field_type="optional")
            if not cur_type["optional"]:
                # An error occured parsing the optional line.
                print "Invalid 'optional' line found in template %s" % enctype
                continue
        elif line.startswith("protected"):
            cur_type["protected"] = __parse_field_ent(parse_ent(line, "protected"),
                                                    field_type="protected")
            if not cur_type["protected"]:
                # An error occured parsing the protected line.
                print "Invalid 'protected' line found in template %s" % enctype
                continue
        elif line.startswith("----"):
            # We're done.
            break
    f.close()
    if not cur_type["required"]:
        print "Failed to find a 'require' line in template %s" % enctype
        return None
    if not cur_type["name"]:
        print "Failed to find a 'name' line in template %s" % enctype
        return None
    else:
        return cur_type

def noneToString(text):
    """ Convert None, "None", or "" to string type "None"

    Used for putting text in a text box.  If the value to put in is 'None',
    the box will be blank.

    """
    if text in (None, ""):
        return "None"
    else:
        return to_unicode(text)

def sanitize_escaped(s):
    """ Sanitize double-escaped unicode strings. """
    lastpos = -1
    while True:
        lastpos = s.find('\\x', lastpos + 1)
        #print lastpos
        if lastpos == -1:
            break
        c = s[lastpos+2:lastpos+4]  # i.e. get the next two characters
        s = s.replace('\\x'+c, chr(int(c, 16)))
    return s

def to_unicode(x):
    """ Attempts to convert a string to utf-8. """
    # If this is a unicode string, encode it and return
    if not isinstance(x, basestring):
        return x
    if isinstance(x, unicode):
        return x.encode('utf-8')

    x = sanitize_escaped(x)

    encoding = locale.getpreferredencoding()
    try:
        ret = x.decode(encoding).encode('utf-8')
    except UnicodeError:
        try:
            ret = x.decode('utf-8').encode('utf-8')
        except UnicodeError:
            try:
                ret = x.decode('latin-1').encode('utf-8')
            except UnicodeError:
                ret = x.decode('utf-8', 'replace').encode('utf-8')
            
    return ret
    
def RenameProcess(new_name):
    """ Renames the process calling the function to the given name. """
    if 'linux' not in sys.platform:
        print 'Unsupported platform'
        return False
    try:
        import ctypes
        from ctypes.util import find_library
        libc = ctypes.CDLL(find_library('c'))
        libc.prctl(15, new_name, 0, 0, 0)
        return True
    except:
        print "rename failed"
        return False
    
def detect_desktop_environment():
    """ Try to determine which desktop environment is in use. 
    
    Choose between kde, gnome, or xfce based on environment
    variables and a call to xprop.
    
    """
    desktop_environment = 'generic'
    if os.environ.get('KDE_FULL_SESSION') == 'true':
        desktop_environment = 'kde'
    elif os.environ.get('GNOME_DESKTOP_SESSION_ID'):
        desktop_environment = 'gnome'
    else:
        try:
            info = getoutput('xprop -root _DT_SAVE_MODE')
            if ' = "xfce4"' in info:
                desktop_environment = 'xfce'
        except (OSError, RuntimeError):
            pass
    return desktop_environment

def get_sudo_cmd(msg, prog_num=0):
    """ Returns a graphical sudo command for generic use. """
    sudo_prog = choose_sudo_prog(prog_num)
    if not sudo_prog: return None
    if re.search("(ktsuss|gksu|gksudo)$", sudo_prog):
        msg_flag = "-m"
    else:
        msg_flag = "--caption"
    return [sudo_prog, msg_flag, msg]

def choose_sudo_prog(prog_num=0):
    """ Try to intelligently decide which graphical sudo program to use. """
    if prog_num:
        return find_path(_sudo_dict[prog_num])
    desktop_env = detect_desktop_environment()
    env_path = os.environ['PATH'].split(":")
    paths = []
    
    if desktop_env == "kde":
        progs = ["kdesu", "kdesudo", "ktsuss"]
    else:
        progs = ["gksudo", "gksu", "ktsuss"]
        
    for prog in progs:
        paths.extend([os.path.join(p, prog) for p in env_path])
        
    for path in paths:
        if os.path.exists(path):
            return path
    return ""

def find_path(cmd):
    """ Try to find a full path for a given file name. 
    
    Search the all the paths in the environment variable PATH for
    the given file name, or return None if a full path for
    the file can not be found.
    
    """
    paths = os.getenv("PATH").split(':')
    if not paths:
        paths = ["/usr/local/sbin", "/usr/local/bin", "/usr/sbin", "/usr/bin",
                 "/sbin", "/bin"]
    for path in paths:
        if os.path.exists(os.path.join(path, cmd)):
            return os.path.join(path, cmd)
    return None

def noneToBlankString(text):
    """ Converts NoneType or "None" to a blank string. """
    if text in (None, "None"):
        return ""
    else:
        return str(text)

def stringToNone(text):
    """ Performs opposite function of noneToString. """
    if text in ("", None, "None"):
        return None
    else:
        return str(text)

def checkboxTextboxToggle(checkbox, textboxes):
    for textbox in textboxes:
        textbox.set_sensitive(checkbox.get_active())

def threaded(f):
    """ A decorator that will make any function run in a new thread. """

    def wrapper(*args, **kwargs):
        t = Thread(target=f, args=args, kwargs=kwargs)
        t.setDaemon(True)
        t.start()

    wrapper.__name__ = f.__name__
    wrapper.__dict__ = f.__dict__
    wrapper.__doc__ = f.__doc__
    wrapper.__module__ = f.__module__

    return wrapper

def timeout_add(time, func, milli=False):
    """ Convience function for running a function on a timer. """
    if hasattr(gobject, "timeout_add_seconds") and not milli:
        return gobject.timeout_add_seconds(time, func)
    else:
        if not milli: time = time * 1000
        return gobject.timeout_add(time, func)

def izip_longest(*args, **kwds):
    """ Implement the itertools.izip_longest method.
    
    We implement the method here because its new in Python 2.6.
    
    """
    # izip_longest('ABCD', 'xy', fillvalue='-') --> Ax By C- D-
    fillvalue = kwds.get('fillvalue')
    def sentinel(counter = ([fillvalue]*(len(args)-1)).pop):
        yield counter()         # yields the fillvalue, or raises IndexError
    fillers = repeat(fillvalue)
    iters = [chain(it, sentinel(), fillers) for it in args]
    try:
        for tup in izip(*iters):
            yield tup
    except IndexError:
        pass

def grouper(n, iterable, fillvalue=None):
    """ Iterate over several elements at once

    "grouper(3, 'ABCDEFG', 'x') --> ABC DEF Gxx"

    """
    args = [iter(iterable)] * n
    return izip_longest(fillvalue=fillvalue, *args)
